# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to select segments according to different criteria such as class member, curvature, diffraction quality
"""
from EMAN2 import EMData
from collections import namedtuple
from scipy import signal
from spring.csinfrastr.csdatabase import SpringDataBase, base, SegmentTable, CtfMicrographTable, HelixTable, \
    CtfFindMicrographTable, CtfTiltMicrographTable
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csreadinput import OptHandler
from sqlalchemy.sql.expression import and_, or_
from tabulate import tabulate
import numpy as np
import shutil


class SegmentSelectPar:
    """
    Class to initiate default dictionary with input parameters including help and range values and 
    status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'
        self.progname = 'segmentselect'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.segmentselect_features = Features()
        self.feature_set = self.segmentselect_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    

    def define_parameters_and_their_properties(self):
        self.feature_set = self.segmentselect_features.set_inp_stack(self.feature_set)
        self.feature_set = self.segmentselect_features.set_out_stack(self.feature_set)
        self.feature_set = self.segmentselect_features.set_spring_path_segments(self.feature_set)
        self.feature_set = self.segmentselect_features.set_database_selected(self.feature_set)
        
        self.feature_set = self.segmentselect_features.set_selection_criteria_from_segment_table(self.feature_set)
        
        
    def define_program_states(self):

        self.feature_set.program_states['extract_segments']='Extract specified classes'


class SegmentSelectPreparation(object):
    """
    * Class that holds functions to extract members from class average stack
    
    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            self.p = self.feature_set.parameters

            self.infile = self.p['Image input stack']
            self.outfile = self.p['Image output stack']
        
            self.spring_db_path = self.p['spring.db file']
            self.db_selected = self.p['Selected database']
            
            self = self.define_selection_parameters_from_segment_table(self, self.p)


    def define_mics_and_helices_selection(self, obj, p):
        obj.mics_selection = p['Micrographs select option']
        obj.mics_in_or_exclude = p['Include or exclude micrographs']
        obj.mics_entries = p['Micrographs list']
        
        obj.helices_selection = p['Helices select option']
        obj.helices_in_or_exclude = p['Include or exclude helices']
        obj.helices_entries = p['Helices list']
        
        return obj


    def define_straightness_selection(self, obj, p):
        obj.straightness_selection = p['Straightness select option']
        obj.straightness_in_or_exclude = p['Include or exclude straight helices']
        obj.curvature_range_perc = p['Persistence length range']
        
        return obj


    def define_defocus_and_astigmatism_selection(self, obj, p):
        obj.defocus_selection = p['Defocus select option']
        obj.defocus_in_or_exclude = p['Include or exclude defocus range']
        obj.defocus_range = p['Defocus range']

        obj.astigmatism_selection = p['Astigmatism select option']
        obj.astigmatism_in_or_exclude = p['Include or exclude astigmatic segments']
        obj.astigmatism_range = p['Astigmatism range']
        
        return obj


    def define_selection_parameters_from_segment_table(self, obj, p):
        obj = self.define_mics_and_helices_selection(obj, p)
        
        obj.segments_selection = p['Segments select option']
        obj.segments_in_or_exclude = p['Include or exclude segments']
        obj.segments_file = p['Segment file']

        obj.classes_selection = p['Classes select option']
        obj.classes_in_or_exclude = p['Include or exclude classes']
        obj.class_type = p['Class type']
        obj.classes_entries = p['Classes list']
        
        obj.persistence_class = p['Persistence class option'] 
        obj.persistence_length = p['Persistence class length in Angstrom']
        obj.class_occupancy_thres = p['Class occupancy threshold']

        obj = self.define_straightness_selection(obj, p)
        
        obj.ccc_layer_selection = p['Layer line correlation select option']
        obj.ccc_layer_in_or_exclude = p['Include or exclude segments based on layer-line correlation']
        obj.ccc_layer_range_perc = p['Correlation layer line range']
        
        obj = self.define_defocus_and_astigmatism_selection(obj, p)
        
        return obj
    

    def prepare_list_from_comma_separated_input(self, entry, filter_criterion):
        """
        * Function to convert comma or semicolon separated entry string to \
        continuous list of classes
        
        #. Input: comma or semicolon separated entry string
        #. Output: list of classes
        #. Usage: list = prepare_list_from_comma_separated_input(inputstring)
        
        >>> from spring.segment2d.segmentselect import SegmentSelect
        >>> ss = SegmentSelect()
        >>> ss.prepare_list_from_comma_separated_input('1 ', 'micrograph')
        [1]
        >>> ss.prepare_list_from_comma_separated_input('1,2', 'class')
        [1, 2]
        >>> input = '1,2, 5-10; 66, 99-101'
        >>> ss.prepare_list_from_comma_separated_input(input, 'class') 
        [1, 2, 5, 6, 7, 8, 9, 10, 66, 99, 100, 101]
        >>> input += '; j'
        >>> ss.prepare_list_from_comma_separated_input(input, 'class') #doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
        ValueError: Specified class numbers could not be interpreted as integer
         numbers, double-check entry.
        """
        classes = []
        error_message = 'Specified {0} numbers could not be interpreted as integer numbers, double-check entry.'.\
        format(filter_criterion)
        for entry in entry.split(';'):
            for each_entry in entry.split(','):
                if each_entry.find('-') >= 1:
                    try:
                        first_class = int(each_entry.split('-')[0])
                        last_class = int(each_entry.split('-')[-1]) + 1
                    except:
                        raise ValueError(error_message)
                    for each_subentry in range(first_class, last_class):
                        classes.append(each_subentry)
                else:
                    try:
                        classes.append(int(each_entry))
                    except:
                        raise ValueError(error_message)
        
        return classes
            

class SegmentSelectFilter(SegmentSelectPreparation):

    def read_segment_selection_from_file(self, segments, segments_selection, segments_in_or_exclude, segment_file):
        if segments_selection:
            sfile = open(segment_file, 'r')
            try:
                segment_ids = [int(each_line) for each_line in sfile.readlines() if not each_line.startswith('#')]
            except:
                msg = 'Content of segment selection file could not be interpreted as integer. Double-check correct ' + \
                'format of segment ids.'
                raise ValueError(msg)
            sfile.close()

            if segments_in_or_exclude == 'include':
                included_segments = [each_segment.stack_id for each_segment in segments 
                                    if each_segment.stack_id in segment_ids]
            elif segments_in_or_exclude == 'exclude':
                included_segments = [each_segment.stack_id for each_segment in segments 
                                    if each_segment.stack_id not in segment_ids]

        elif not segments_selection:
            included_segments = [each_segment.stack_id for each_segment in segments]
                    
        excluded_seg_count = len(segments) - len(set(included_segments))
        
        return included_segments, excluded_seg_count


    def compute_local_average_from_measurements(self, segment_measurements, window_size):
        """
        >>> from spring.segment2d.segmentselect import SegmentSelect
        >>> SegmentSelect().compute_local_average_from_measurements(np.arange(10), 4)
        array([ 1.  ,  1.  ,  1.5 ,  2.5 ,  3.5 ,  4.5 ,  5.5 ,  6.5 ,  7.5 ,  8.25])
        >>> rand_series = [0.33, -0.84, -0.55, 0.27, 0.26, -0.09, -0.85, -0.18, 0.71, 0.07]
        >>> SegmentSelect().compute_local_average_from_measurements(rand_series, 5)
        array([-0.49 , -0.326, -0.106, -0.19 , -0.192, -0.118, -0.03 , -0.068,
               -0.036,  0.276])
        >>> SegmentSelect().compute_local_average_from_measurements(np.arange(10), 12)
        array([ 2.5,  2.5,  2.7,  3.1,  3.7,  4.5,  5.4,  6.1,  6.6,  6.9])
        >>> SegmentSelect().compute_local_average_from_measurements(rand_series, 2) #doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
          File '/System/Library/Frameworks/Python.framework/Versions/2.6/lib/python2.6/doctest.py', 
              line 1231, in __run
            compileflags, 1) in test.globs
          File '<doctest segment.SegmentSelect.compute_local_average_from_measurements[4]>', 
              line 1, in <module>
            SegmentSelect().compute_local_average_from_measurements(rand_series, 2)
          File 'spring/segment2d/segment.py', line 994, in 
              compute_local_average_from_measurements
            raise ValueError, msg
        ValueError: Please choose larger averaging distance or decrease step 
        size of segmentation to allow smaller local averaging. Otherwise turn off 
        local averaging option.

        """
        if window_size < 3:
            msg = 'Please choose larger averaging distance or decrease step size of segmentation to allow smaller ' + \
            'local averaging. Otherwise turn off local averaging option.'
            raise ValueError(msg)
        window_function = np.zeros(len(segment_measurements))
        if window_size > len(segment_measurements):
            window_size = len(segment_measurements)
        for each_index in range(window_size):
            window_function[each_index] = 1.0
            
        window_function = (np.roll(window_function, (len(segment_measurements) - window_size)/2))/window_size
        
        measurements_with_borders = np.r_[segment_measurements[window_size-1:0:-1], segment_measurements,
        segment_measurements[-1:-window_size:-1]]
        
        locally_averaged_measurements = signal.fftconvolve(measurements_with_borders, window_function, mode='same')

        locally_averaged_measurements = locally_averaged_measurements[window_size-1:-(window_size - 1)]
        
        return locally_averaged_measurements
            
            
    def determine_included_stack_ids_according_to_class_persistence(self, segments, class_in_or_exclude,
    persistence_length, class_list, helices, class_occupancy_thres=0.5, class_attr='class_id'):
        """
        >>> from spring.segment2d.segmentselect import SegmentSelect
        >>> class_list = [1, 2]
        >>> stack_ids = range(10, 30)
        >>> class_ids = 6 * [0] + 6 * [1] + 8 * [2] 
        >>> helix_ids = 10 * [11] + 10 * [12]
        >>> distances = 2 * np.linspace(0, 1400, 10).tolist()
        >>> segment_nt = namedtuple('seg', 'stack_id class_id, helix_id, distance_from_start_A')
        >>> segments = [segment_nt._make(each_info) for each_info in zip(stack_ids, class_ids, helix_ids, distances)]
        >>> helix_nt = namedtuple('hel', 'id')
        >>> helices = [helix_nt(each_helix_id) for each_helix_id in set(helix_ids)]
        >>> s = SegmentSelect()
        >>> s.determine_included_stack_ids_according_to_class_persistence(segments, 'include', 700, class_list, helices)
        [16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29]
        >>> s.determine_included_stack_ids_according_to_class_persistence(segments, 'exclude', 700, class_list, helices)
        [10, 11, 12, 13, 14, 15]
        >>> helix_ids = 10 * [15] + 10 * [12]
        >>> segments = [segment_nt._make(each_info) for each_info in zip(stack_ids, class_ids, helix_ids, distances)]
        >>> s.determine_included_stack_ids_according_to_class_persistence(segments, 'include', 700, class_list, helices)
        [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
        >>> class_list = [0, 1]
        >>> class_ids = 1 * [0] + 1 * [1] + 8 * [2] + 1 * [0] + 1 * [1] + 8 * [2] 
        >>> segments = [segment_nt._make(each_info) for each_info in zip(stack_ids, class_ids, helix_ids, distances)]
        >>> s.determine_included_stack_ids_according_to_class_persistence(segments, 'include', 700, class_list, helices)
        []
        """
        stack_ids = np.array([each_segment.stack_id for each_segment in segments])
        class_ids = np.array([getattr(each_segment, class_attr) for each_segment in segments])
        helix_ids = np.array([each_segment.helix_id for each_segment in segments])
        distances = np.array([each_segment.distance_from_start_A for each_segment in segments])
        classes_present = np.in1d(class_ids, np.array(class_list))
        if class_in_or_exclude == 'exclude':
            classes_present = np.invert(classes_present)

        classes_percent = classes_present.astype(float)
        finally_selected = np.array([])
        log_ids = np.array([])
        hel_ids = np.array([])
        occ = np.array([])
        
        to_be_logged = []

        for each_helix in helices:
            classes_helix = classes_percent[helix_ids == each_helix.id]
            stepsize = (np.diff(distances[helix_ids == each_helix.id]))
            if len(stepsize) > 0:
                avg_count = int(round(persistence_length / abs(np.mean(stepsize))))
                avg_classes = self.compute_local_average_from_measurements(classes_helix, avg_count)
                selected = avg_classes > class_occupancy_thres
                if len(selected[selected == True]) <= 3:
                    selected = np.zeros(len(selected), dtype=bool)
                finally_selected = np.append(finally_selected, selected)
    
                to_be_logged += zip(stack_ids[helix_ids == each_helix.id], len(avg_classes) * [each_helix.id], avg_classes)
    
                log_ids = np.append(log_ids, stack_ids[helix_ids == each_helix.id])
                hel_ids = np.append(hel_ids, len(avg_classes) * [each_helix.id])
                occ = np.append(occ, avg_classes)
        
        finally_selected = finally_selected.astype(bool)
        included_segments_classes = stack_ids[finally_selected].tolist()

        msg = tabulate(to_be_logged, headers=['stack_id', 'helix_id', 'class_occupancy'])
        self.log.ilog(msg)

        return included_segments_classes


    def filter_segments_by_classes_and_persistence(self, session, segments, classes_selection, class_in_or_exclude,
    class_entries, persistence_class, persistence_length, class_occupancy_thres, class_type):
        class_attr = class_type
            
        if classes_selection:
            class_list = self.prepare_list_from_comma_separated_input(class_entries, 'class')
            if class_in_or_exclude == 'include' and not persistence_class:
                included_segments_classes = [each_segment.stack_id for each_segment in segments 
                                             if getattr(each_segment, class_attr) in class_list]
                    
            elif class_in_or_exclude == 'exclude' and not persistence_class:
                included_segments_classes = [each_segment.stack_id for each_segment in segments 
                                             if getattr(each_segment, class_attr) not in class_list]

            if persistence_class:
                helices = session.query(HelixTable).order_by(HelixTable.id).all()

                included_segments_classes = self.determine_included_stack_ids_according_to_class_persistence(segments,
                class_in_or_exclude, persistence_length, class_list, helices, class_occupancy_thres, class_attr)
                    
        elif not classes_selection:
            included_segments_classes = [each_segment.stack_id for each_segment in segments]
                    
        excluded_count = len(segments) - len(set(included_segments_classes))
        
        return included_segments_classes, excluded_count


    def filter_segments_by_entry_string(self, segments, attr_id, attr_selection, attr_in_or_exclude,
    attr_entries, filter_criterion):

        if attr_selection:
            attr_list = self.prepare_list_from_comma_separated_input(attr_entries, filter_criterion)
            if attr_in_or_exclude == 'include':
                included_segments_classes = [each_segment.stack_id for each_segment in segments 
                                             if getattr(each_segment, attr_id) in attr_list]
                    
            elif attr_in_or_exclude == 'exclude':
                included_segments_classes = [each_segment.stack_id for each_segment in segments 
                                             if getattr(each_segment, attr_id) not in attr_list]

        elif not attr_selection:
            included_segments_classes = [each_segment.stack_id for each_segment in segments]
                    
        excluded_count = len(segments) - len(set(included_segments_classes))
        
        return included_segments_classes, excluded_count
    
        
    def get_excluded_count(self, session, included_segments_classes):
        segment_count = session.query(SegmentTable).order_by(SegmentTable.stack_id).count()
        excluded_count = segment_count - len(set(included_segments_classes))
        
        return excluded_count

    
    def filter_segments_by_property(self, session, segment_table_property, property_selection, property_in_or_exclude,
    property_range):
        included_segments_property = []
        if property_selection:
            if property_in_or_exclude == 'include':
                included_segments = session.query(SegmentTable).\
                filter(and_(segment_table_property >= property_range[0], 
                            segment_table_property <= property_range[1])).all()
                            
                included_segments_property = [each_segment.stack_id for each_segment in included_segments]

            elif property_in_or_exclude == 'exclude':
                included_segments = session.query(SegmentTable).\
                filter(or_(segment_table_property < property_range[0], 
                           segment_table_property > property_range[1])).all()
                           
                included_segments_property = [each_segment.stack_id for each_segment in included_segments]

        elif not property_selection:
            included_segments = session.query(SegmentTable).all()
            included_segments_property = [each_segment.stack_id for each_segment in included_segments]
            
        excluded_count = self.get_excluded_count(session, included_segments_property)
        
        return included_segments_property, excluded_count
    

    def filter_by_properties_that_keep_helices_together(self, obj, session, segments):
        included_segments_mics, excluded_mics_count = self.filter_segments_by_entry_string(segments, 'mic_id', 
        obj.mics_selection, obj.mics_in_or_exclude, obj.mics_entries, 'micrograph')

        included_segments_helices, excluded_helix_count = self.filter_segments_by_entry_string(segments, 
        'helix_id', obj.helices_selection, obj.helices_in_or_exclude, obj.helices_entries, 'helix')

        included_segments_curve, excluded_curvature_count = self.filter_segments_by_property(session, 
        SegmentTable.curvature, obj.straightness_selection, obj.straightness_in_or_exclude, obj.curvature_range)

        included_segments_defocus, excluded_defocus_count = self.filter_segments_by_property(session, 
        SegmentTable.avg_defocus, obj.defocus_selection, obj.defocus_in_or_exclude, obj.defocus_range)

        included_segments_astigmatism, excluded_astig_count = self.filter_segments_by_property(session, 
        SegmentTable.astigmatism, obj.astigmatism_selection, obj.astigmatism_in_or_exclude, obj.astigmatism_range)

        return included_segments_mics, included_segments_helices, included_segments_curve, included_segments_defocus,\
        included_segments_astigmatism, excluded_mics_count, excluded_helix_count, excluded_curvature_count, \
        excluded_defocus_count, excluded_astig_count


    def filter_by_property_that_take_helices_apart(self, obj, session, segments):
        included_segments_classes, excluded_class_count = self.filter_segments_by_classes_and_persistence(session, 
        segments, obj.classes_selection, obj.classes_in_or_exclude, obj.classes_entries, obj.persistence_class, 
        obj.persistence_length, obj.class_occupancy_thres, obj.class_type)

        included_segments_ccc_layer, excluded_layer_count = self.filter_segments_by_property(session, 
        SegmentTable.ccc_layer, obj.ccc_layer_selection, obj.ccc_layer_in_or_exclude, obj.ccc_layer_range)

        return included_segments_classes, included_segments_ccc_layer, excluded_class_count, excluded_layer_count


    def check_whether_all_segments_were_excluded(self, combined_included_segments, included_segments_mics,
    included_segments_helices, included_segments_classes, included_segments_curve, included_segments_defocus,
    included_segments_astigmatism, included_segments_ccc_layer):
        """
        >>> from spring.segment2d.segmentselect import SegmentSelect
        >>> s = SegmentSelect()
        >>> s.check_whether_all_segments_were_excluded([], [0], [0], [0], [0], [0], [0], [0])#doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
            ...
        ValueError: All segments were excluded as a result of stringent 
        selection criteria. Please, use less stringent criteria.
        >>> s.check_whether_all_segments_were_excluded([], [], [0], [0], [0], [0], [0], [0])#doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
            ...
        ValueError: All segments were excluded because of 'micrograph' 
        selection criteria. Please, use less stringent selection criteria 
        for micrographs.
        >>> s.check_whether_all_segments_were_excluded([], [0], [], [], [0], [0], [0], [0])#doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
            ...
        ValueError: All segments were excluded because of 'helix', 
        'class' selection criteria. Please, use less stringent 
        selection criteria for helices, classes.
        """
        keyword = []
        if included_segments_mics == []:
            keyword.append(('micrograph', 'micrographs'))
        if included_segments_helices == []:
            keyword.append(('helix', 'helices'))
        if included_segments_classes == []:
            keyword.append(('class', 'classes'))
        if included_segments_curve == []:
            keyword.append(('straightness', 'straightness of helix'))
        if included_segments_defocus == []:
            keyword.append(('defocus', 'defocus'))
        if included_segments_astigmatism == []:
            keyword.append(('astigmatism', 'astigmatism'))
        if included_segments_ccc_layer == []:
            keyword.append(('layer-line correlation', 'layer-line correlation'))
        if keyword != []:
            keyword_one, keyword_two = zip(*keyword)
            error_message = 'All segments were excluded because of \'{0}\' '.format('\', \''.join(keyword_one)) + \
                'selection criteria. Please, use less stringent selection criteria for {0}. '.format(', '.join(keyword_two))
            raise ValueError(error_message)
        
        if combined_included_segments == []:
            message = 'All segments were excluded as a result of stringent selection criteria. Please, use less ' + \
            'stringent criteria.'
            raise ValueError(message)
        
        
    def summarize_counts(self, included_segments, excluded_seg_count, included_segments_mics, included_segments_helices,
    included_segments_defocus, included_segments_astigmatism, excluded_mics_count, excluded_helix_count,
    excluded_defocus_count, excluded_astig_count, included_segments_curve, included_segments_classes,
    included_segments_ccc_layer, excluded_class_count, excluded_curvature_count, excluded_layer_count):
                
        combined_included_segments = list(set(included_segments_curve).intersection(included_segments,
        included_segments_mics, included_segments_helices, included_segments_classes, included_segments_defocus,
        included_segments_astigmatism, included_segments_ccc_layer))
        
        self.check_whether_all_segments_were_excluded(combined_included_segments, included_segments_mics,
        included_segments_helices, included_segments_classes, included_segments_curve, included_segments_defocus,
        included_segments_astigmatism, included_segments_ccc_layer)
        
        excluded_counts = namedtuple('excluded_counts', 'seg_count mic_count helix_count class_count ' + \
                                     'curvature_count defocus_count astig_count layer_cc_count')
        
        excluded_segment_counts = excluded_counts(excluded_seg_count, excluded_mics_count, excluded_helix_count,
        excluded_class_count, excluded_curvature_count, excluded_defocus_count, excluded_astig_count,
        excluded_layer_count)
        
        combined_included_segments.sort()
        
        return excluded_segment_counts, combined_included_segments

        
    def filter_non_orientation_parameters_based_on_selection_criteria(self, obj, spring_db='spring.db',
    keep_helices_together=False):
        session = SpringDataBase().setup_sqlite_db(base, spring_db)
        
        segments = session.query(SegmentTable).order_by(SegmentTable.stack_id).all()
        included_segments, excluded_seg_count = self.read_segment_selection_from_file(segments, obj.segments_selection,
        obj.segments_in_or_exclude, obj.segments_file)

        included_segments_mics, included_segments_helices, included_segments_curve, included_segments_defocus, \
        included_segments_astigmatism, excluded_mics_count, excluded_helix_count, excluded_curvature_count, \
        excluded_defocus_count, excluded_astig_count = \
        self.filter_by_properties_that_keep_helices_together(obj, session, segments)
        
        included_segments_classes, included_segments_ccc_layer, excluded_class_count, excluded_layer_count = \
        self.filter_by_property_that_take_helices_apart(obj, session, segments)

        excluded_segment_counts, combined_included_segments = self.summarize_counts(included_segments,
        excluded_seg_count, included_segments_mics, included_segments_helices, included_segments_defocus,
        included_segments_astigmatism, excluded_mics_count, excluded_helix_count, excluded_defocus_count,
        excluded_astig_count, included_segments_curve, included_segments_classes, included_segments_ccc_layer,
        excluded_class_count, excluded_curvature_count, excluded_layer_count)
        
        if keep_helices_together:
            helix_ids = list(set([each_segment.helix_id for each_segment in segments 
                         if each_segment.stack_id in combined_included_segments]))
            
            combined_included_segments = [each_segment.stack_id for each_segment in segments 
                                          if each_segment.helix_id in helix_ids]
            
            combined_included_segments.sort()

        msg = 'The selection resulted in the following number of included and excluded segments:\n'
        msg += tabulate(zip(['included', 'excluded'], 
                       [str(len(combined_included_segments)), str(len(segments) - len(combined_included_segments))]), 
                       ['total', str(len(segments))])

        msg += '\nThe number of segments that were excluded listed by criteria:\n'
        msg += tabulate(zip(list(excluded_segment_counts._fields), list(excluded_segment_counts)))
        msg += '\nThe following {0} segments are included in the selection:\n{1}'.\
        format(len(combined_included_segments), ', '.join([str(each_seg) for each_seg in combined_included_segments]))
        self.log.ilog(msg)

        return combined_included_segments, excluded_segment_counts 
    
    
class SegmentSelect(SegmentSelectFilter):

    def remove_all_previous_entries(self, session, column_name):
        column = session.query(column_name).all()
        for each_item in column:
            session.delete(each_item)
        
        return session
    

    def copy_segments_onto_output_stack(self, segment_ids):
        segment = EMData()
        for each_new_id, each_seg_id in enumerate(segment_ids):
            segment.read_image(self.infile, each_seg_id)
            segment.write_image(self.outfile, each_new_id)
            self.log.ilog('Segment {0} included in the output stack: new segment {1}.'.format(each_seg_id, each_new_id))
            

    def add_filtered_segments(self, segment_ids, session, updated_session):

        segments = session.query(SegmentTable).all()
        columns = SpringDataBase().get_columns_from_table(SegmentTable)
        each_updated_seg_id = 0
        for each_segment in segments:
            if each_segment.stack_id in segment_ids:
                data = SpringDataBase().get_data_from_entry(columns, each_segment)
                data['stack_id'] = each_updated_seg_id
                each_updated_seg_id += 1
                data.__delitem__('id')
                updated_session.add(SegmentTable(**data))
            
        return updated_session
    
        
    def update_ccc_layer_in_each_helix(self, updated_session, each_helix):
        segments = updated_session.query(SegmentTable).filter(SegmentTable.helix_id == each_helix.id)
        ccc_layer = [each_segment.ccc_layer for each_segment in segments]
        if ccc_layer != []:
            if ccc_layer[0] is not None:
                each_helix.avg_ccc_layer = np.mean(ccc_layer)
        
        return updated_session
    

    def update_curvature_in_each_helix(self, updated_session, each_helix):
        segments = updated_session.query(SegmentTable).filter(SegmentTable.helix_id == each_helix.id)
        curvature = [each_segment.curvature for each_segment in segments]
        if curvature != []:
            if curvature[0] is not None:
                each_helix.avg_curvature = np.mean(curvature)
        
        return updated_session


    def update_inplane_angle_in_each_helix(self, updated_session, each_helix):
        segments = updated_session.query(SegmentTable.inplane_angle).filter(SegmentTable.helix_id == each_helix.id)
        inplane_angle = [each_segment.inplane_angle for each_segment in segments]
        if inplane_angle != []:
            if inplane_angle[0] is not None:
                each_helix.avg_inplane_angle = np.mean(inplane_angle)
        
        return updated_session
    

    def update_records_in_helix(self, updated_session):
        helices = updated_session.query(HelixTable).all()
        
        for each_helix in helices:
            updated_session = self.update_ccc_layer_in_each_helix(updated_session, each_helix)
            updated_session = self.update_curvature_in_each_helix(updated_session, each_helix)
            updated_session = self.update_inplane_angle_in_each_helix(updated_session, each_helix)
            
            updated_session.merge(each_helix)
            
        return updated_session
            
    
    def write_out_new_database_for_filtered_stack(self, segment_ids):
        session = SpringDataBase().setup_sqlite_db(base)
        updated_session = SpringDataBase().setup_sqlite_db(base, self.db_selected)
    
        updated_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(CtfMicrographTable,
        updated_session, session)

        updated_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(CtfFindMicrographTable,
        updated_session, session)

        updated_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(CtfTiltMicrographTable,
        updated_session, session)

        updated_session = SpringDataBase().copy_all_table_data_from_one_session_to_another_session(HelixTable,
        updated_session, session)

        updated_session = self.add_filtered_segments(segment_ids, session, updated_session)
        
        updated_session.commit()
        
        updated_session = self.update_records_in_helix(updated_session)
        
        updated_session.commit()
        
        
    def convert_relative_range_to_absolute_range_values(self, pers_length, curvature_range_percent):
        """
        >>> from spring.segment2d.segmentselect import SegmentSelect
        >>> s = SegmentSelect()
        >>> s.convert_relative_range_to_absolute_range_values(np.arange(10), [80, 100])
        (7, 9)
        >>> s.convert_relative_range_to_absolute_range_values(np.arange(10), [0, 20])
        (0, 1)
        >>> s.convert_relative_range_to_absolute_range_values(np.arange(10), [60, 80])
        (5, 7)
        """
        lower_bound, upper_bound = curvature_range_percent
        lower_bound_id = int(len(pers_length) * lower_bound / 100.0 - 0.5)
        upper_bound_id = int(len(pers_length) * upper_bound / 100.0 - 0.5)
        
        sorted_pers_length = np.sort(pers_length)

        curvature_range_vals = (sorted_pers_length[lower_bound_id], sorted_pers_length[upper_bound_id])
        
        return curvature_range_vals


    def convert_curvature_ccc_layer_range(self, spring_db, straightness_selection, curvature_range,
    ccc_layer_selection=None, ccc_layer_range=None):
        session = SpringDataBase().setup_sqlite_db(base, spring_db)
        segments = session.query(SegmentTable).all()
        
        if straightness_selection:
            pers_length = np.array([each_segment.curvature for each_segment in segments])
            curvature_range = self.convert_relative_range_to_absolute_range_values(pers_length, curvature_range)

        if ccc_layer_selection:
            ccc_layer = np.array([each_segment.ccc_layer for each_segment in segments])
            ccc_layer_range = self.convert_relative_range_to_absolute_range_values(ccc_layer, ccc_layer_range)
        
        return curvature_range, ccc_layer_range


    def extract_segments(self):
        self.log.fcttolog()
        shutil.copy(self.spring_db_path, 'spring.db')
        
        self.curvature_range, self.ccc_layer_range = self.convert_curvature_ccc_layer_range('spring.db',
        self.straightness_selection, self.curvature_range_perc, self.ccc_layer_selection, self.ccc_layer_range_perc)
        
        combined_included_segments, excluded_segment_counts = \
        self.filter_non_orientation_parameters_based_on_selection_criteria(self, keep_helices_together=False)
        
        self.log.plog(10)
        if combined_included_segments != []:
            self.write_out_new_database_for_filtered_stack(combined_included_segments)
            self.log.plog(20)
            self.copy_segments_onto_output_stack(combined_included_segments)
            self.log.plog(90)
        else:
            self.log.wlog('No segments passed the selection criteria. No sensible selection possible. Please ' + \
                          'revise criteria.')
            
        self.log.endlog(self.feature_set)
        
def main():
    # Option handling
    parset = SegmentSelectPar()
    
    mergeparset = OptHandler(parset)

    ######## Program
    stack = SegmentSelect(mergeparset)
    stack.extract_segments()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Author: Carsten Sachse 21-Sep-2010
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from PyQt4.QtCore import QSignalMapper, SIGNAL, SLOT, Qt, QThread, pyqtSignal, QSemaphore
from PyQt4.QtGui import QApplication, QWidget, QGridLayout, QVBoxLayout, QLineEdit, QDoubleSpinBox, QCheckBox, QLabel, \
    QPushButton, QProgressBar, QStatusBar, QFrame, QTextBrowser, QMessageBox, QFileDialog, QFont, QCompleter, QComboBox, \
    QListWidget, QAbstractItemView, QTabWidget
from collections import OrderedDict
from functools import partial
from glob import glob
from math import log10
from random import random
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import NameGenerator
from spring.csinfrastr.csproductivity import ExtLauncher
from textwrap import wrap
from time import sleep
import os
"""
* Several classes for Graphical User Interface'
"""


class LauncherGui(QThread):
    def __init__(self, parent=None):
        QThread.__init__(self, parent)
        self._run_semaphore = QSemaphore(1)
        
    def pass_program_command_line(self, progname, dirprogname):
        self.progname = progname
        self.dirprogname = dirprogname

    def run(self):
        if self._run_semaphore.available() == 0:
            self._run_semaphore.release(1)

        self.proc = ExtLauncher()
        self.proc.qlaunch(self.progname, self.dirprogname)
        self.proc.proc.waitForFinished(-1)
    
        self.stop()
            
    def stop(self):
        self.proc.finished()
        self._run_semaphore.acquire(1)
        
    def __del__(self):
        self.exiting = True
        self.wait()
        
        
class ProgressBarUpdater(QThread):
    #: this signal is emitted whenever the barUpdater has performed a single
    #  step
    progress = pyqtSignal(int)
    #: this signal publishes the minimum amount of required steps
    minimumChanged = pyqtSignal(int)
    #: this signal publishes the maximum amount of required steps
    maximumChanged = pyqtSignal(int)
    #: this signal gets the logfile while it is written
    log = pyqtSignal(str)
    #: this signal transmits the error
    err = pyqtSignal(str)

    def __init__(self, pardict=None, statedict = None, progname=None, parent=None):
        """
        this semaphore is used to stop the thread from outside.  As long
        as the thread is permitted to run, it has yet a single resource
        available.  If the thread is stopped, this resource is acquired,
        and the thread breaks its counting loop at the next iteration.
        """
        QThread.__init__(self, parent)
        self.maximum = 100.0
        self.parameters = pardict
        self.program_states = statedict
        self.progname = progname
        
        self._run_semaphore = QSemaphore(1)

    def readLogncount(self, logfile = None):
        if logfile is None: logfile = self.logfile
        fh = open(logfile)
        fh.seek(0 , os.SEEK_END)  # Go to the end of the file
        hasended = False
        state = 0
        while True:
            QApplication.processEvents()
            line = fh.readline()
            if not line:
                sleep(0.1)    # Sleep briefly
                continue
            
            self.log.emit(line.strip())
            if line.find(':progress state:') >= 0:
                part_line = line.split(':')[-1]
                percent = int(part_line.split('%')[0].strip())
                if percent * 100.0 / (self.maximum) > state: 
                    state = percent * 100.0 / (self.maximum)
                    self.progress.emit(state)
            if line.find('End of program {0}'.format(self.progname[0:-4])) >= 0:
                self.progress.emit(100)
                hasended = True
                self.err.emit('Status: Completed.')
                break
            if line.find('Error:') >= 0 and line.find('TypeError:') < 0:
                spliterr = line.split(':')
                self.err.emit('Status: Interrupted ({0}).'.format(spliterr[0].strip(':')))
                hasended = True
                break
            for event in self.program_states:
                if line.find(':' + event) >= 0:
                    self.err.emit('Status: {0}'.format(self.program_states[event]))
                    break
            # check, if the thread is still permitted to run.  This test
            # fails, if the single resource of this semaphore is acquired,
            # which is done by the stop() method, just to stop work at
            # precisly this point
            if self._run_semaphore.available() == 0:
                # release the resource again to enable a restart of this barUpdater
                self._run_semaphore.release(1)
                # break the loop.  A real barUpdater would typically roll back
                # his work to leave the state of the application unchanged,
                # because the work was forcibly interrupted.
                break
            if hasended is True:
                break
        fh.close()

    def run(self):
        """
        published our minimum and maximum values.  A real barUpdater thread
        would typically calculate the number of required steps

        update maximum by multiplying by number of files and number images on stack
        """
        
        self.minimumChanged.emit(1)
        self.maximumChanged.emit(self.maximum)
        
        while True:
            if os.path.exists(self.logfile): 
                break
        self.readLogncount(self.logfile)
        

    def stop(self):
        """
        acquire the single resource of our run semaphore.  No resources
        are now available anymore, so next time, run() checks the number
        of available resources, it sees none and returns.
        """
        lfile = open(self.logfile, 'a')
        lfile.write('%s was interrupted by the GUI' %self.progname)
        lfile.close()

        self._run_semaphore.acquire(1)
        
    def __del__(self):
        self.exiting = True
#        self.wait()
        
        
class NumbersOptionsGuiWindow(object):

    def appendWidgetToLevel(self, *widgets):
        for widget in widgets:
            if self.level[self.name] in ['beginner']:
                self.beginners.append(widget)
            elif self.level[self.name] in ['intermediate']:
                self.intermediates.append(widget)
            elif self.level[self.name] in ['expert']:
                self.experts.append(widget)

    def compute_number_of_digits_from_properties(self, obj, name):
        obj.digits = log10(obj.properties[name].step)
        if obj.digits > 0:
            obj.digits = 0
        else:
            obj.digits = abs(obj.digits)
        return obj


    def setToolTipIncludingLineBreak(self, hint_txt):
        return self.thisWidget.setToolTip('\n'.join(wrap(hint_txt)))
    

    def convert_angstrom_string(self, name):
        """
        >>> from spring.csinfrastr.csgui import NumbersOptionsGuiWindow 
        >>> n = NumbersOptionsGuiWindow()
        >>> n.convert_angstrom_string('This is my Angstrom')
        'This is my \\xc5'
        >>> n.convert_angstrom_string('This is my Angstrom site')
        'This is my \\xc5 site'
        >>> n.convert_angstrom_string('This is my first Angstrom, second Angstrom and third Angstrom')
        'This is my first \\xc5, second \\xc5 and third \\xc5'
        """
        split_angstrom = name.split('Angstrom')
        lname = u'\xc5'.encode('latin-1').join(split_angstrom)
            
        return lname
    

    def prepareSpinBox(self):
        if self.name.find('Angstrom') >= 0: 
            lname = self.convert_angstrom_string(self.name)
        else:
            lname = self.name
        self.label.setText('%s' %(lname))
        self = self.compute_number_of_digits_from_properties(self, self.name)
        self.thisWidget = QDoubleSpinBox()

        self.setToolTipIncludingLineBreak('{0} Example (default): \'{1}\''.format(self.hints[self.name], 
        self.default_params[self.name]))

        self.appendWidgetToLevel(self.label)
        

    def insertOneSpinBox(self):
        self.widgets[self.name]=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 2, 1, 2)
        self.thisWidget.setRange(self.properties[self.name].minimum, self.properties[self.name].maximum)
        self.thisWidget.setSingleStep(self.properties[self.name].step)
        self.thisWidget.setDecimals(self.digits)
        self.appendWidgetToLevel(self.thisWidget)
        

    def insertDoubleSpinBox(self):
        self.widgets[self.name + '1']=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 2)
        self.thisWidget.setRange(self.properties[self.name].minimum, self.properties[self.name].maximum)
        self.thisWidget.setSingleStep(self.properties[self.name].step)
        self.thisWidget.setDecimals(self.digits)
        self.appendWidgetToLevel(self.thisWidget)

        self.thisWidget = QDoubleSpinBox()
        self.widgets[self.name + '2']=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 3)
        self.thisWidget.setRange(self.properties[self.name].minimum, self.properties[self.name].maximum)
        self.thisWidget.setSingleStep(self.properties[self.name].step)
        self.thisWidget.setDecimals(self.digits)

        self.setToolTipIncludingLineBreak('{0} Example (default): \'{1}\''.format(self.hints[self.name], 
        self.default_params[self.name]))

        self.appendWidgetToLevel(self.thisWidget)
    
    
    def insertCheckBox(self):
        self.thisWidget = QCheckBox()
        self.widgets[self.name]=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 2, 1, 2)
        self.thisWidget.setText('%s' %(self.name))

        self.setToolTipIncludingLineBreak('{0} Example (default): \'{1}\''.format(self.hints[self.name], 
        self.default_params[self.name]))

        self.appendWidgetToLevel(self.thisWidget)


class FilesGuiWindow(object):
    def insertLineEdit(self):
        self.label.setText('%s' %(self.name))
        self.thisWidget = QLineEdit()
        self.widgets[self.name]=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 2, 1, 2)

        self.setToolTipIncludingLineBreak('{0} Example (default): \'{1}\''.format(self.hints[self.name], 
        self.default_params[self.name]))

        self.appendWidgetToLevel(self.label, self.thisWidget)

    def setLineEditAccordingToFileProperties(self):
        if self.properties[self.name].ftype == 'getFile':
            self.button.setText('Input')
            self.connect(self.button, SIGNAL('clicked()'), self.getfilemapper, SLOT('map()'))
            self.getfilemapper.setMapping(self.button, str(self.cellrow))
            self.button.setToolTip('Browse file')
            self.gridlayout.addWidget(self.button, self.cellrow + 1, 4)
        elif self.properties[self.name].ftype == 'getFiles':
            self.button.setText('Input')
            self.connect(self.button, SIGNAL('clicked()'), self.getfilesmapper, SLOT('map()'))
            self.getfilesmapper.setMapping(self.button, str(self.cellrow))
            self.button.setToolTip('Browse file')
            self.gridlayout.addWidget(self.button, self.cellrow + 1, 4)
        elif self.properties[self.name].ftype == 'saveFile':
            self.addOpenButtonToViewResult()
        elif self.properties[self.name].ftype == 'getDir':
            self.button.setText('Directory')
            self.connect(self.button, SIGNAL('clicked()'), self.getdirmapper, SLOT('map()'))
            self.getdirmapper.setMapping(self.button, str(self.cellrow))
            self.button.setToolTip('Browse directory')
            self.gridlayout.addWidget(self.button, self.cellrow + 1, 4)
        
        if self.properties[self.name].ftype in ['getFile', 'getFiles', 'getDir']:
            self.appendWidgetToLevel(self.button)
            self.widgets[self.name + 'button']=self.button
        

    def addOpenButtonToViewResult(self):
        self.openbutton.setText('Open result')
        self.openbutton.setToolTip('Open output file')
        self.openbutton.setEnabled(False)
        self.connect(self.openbutton, SIGNAL('clicked()'), self.openbuttonmapper, SLOT('map()'))
        self.openbuttonmapper.setMapping(self.openbutton, str(self.cellrow))
        self.widgets[self.name + 'openbutton'] = self.openbutton
        self.gridlayout.addWidget(self.openbutton, self.cellrow + 1, 4)
        self.appendWidgetToLevel(self.openbutton)
        

    def connectCheckBoxToEnableOther(self):
        """
        * Function connect widget to Checkbox and dis and enable upon click 
        """
        if self.name in self.relatives:
            if type(self.parameters[self.name]) is tuple and type(self.parameters[self.relatives[self.name][0]]) is \
            bool:
                self.connect(self.widgets[self.relatives[self.name][1]], SIGNAL('toggled(bool)'),
                              self.widgets[self.name + '1'], SLOT('setEnabled(bool)'))
                self.widgets[self.name + '1'].setEnabled(False)
                self.connect(self.widgets[self.relatives[self.name][0]], SIGNAL('toggled(bool)'), 
                             self.widgets[self.name + '2'], SLOT('setEnabled(bool)'))
                self.widgets[self.name + '2'].setEnabled(False)
                for widg in [self.label]:
                    self.connect(self.widgets[self.relatives[self.name][1]], SIGNAL('toggled(bool)'), 
                                 widg, SLOT('setEnabled(bool)'))
                    widg.setEnabled(False)
                    self.connect(self.widgets[self.relatives[self.name][0]], SIGNAL('toggled(bool)'), 
                                 widg, SLOT('setEnabled(bool)'))
                    widg.setEnabled(False)
            elif type(self.parameters[self.name]) is not tuple and type(self.parameters[self.relatives[self.name]]) is \
            bool:
                for widg in [self.thisWidget, self.button, self.label]:
                    self.connect(self.widgets[self.relatives[self.name]], SIGNAL('toggled(bool)'), 
                                 widg, SLOT('setEnabled(bool)'))
                    widg.setEnabled(False)


    def insertComboBox(self):
        self.label.setText('{0}'.format(self.name))
        self.thisWidget = QComboBox()
        choices = [each_choice.capitalize() for each_choice in self.properties[self.name].choices]
        self.thisWidget.addItems(choices)
        self.widgets[self.name]=self.thisWidget
        self.gridlayout.addWidget(self.thisWidget, self.cellrow + 1, 2, 1, 2)

        self.setToolTipIncludingLineBreak('{0} Example (default): \'{1}\''.format(self.hints[self.name], 
        self.default_params[self.name]))

        self.appendWidgetToLevel(self.label, self.thisWidget)
        

class GuiWindow(NumbersOptionsGuiWindow, FilesGuiWindow):

    def setupUi(self, Window):
        self.setWindowTitle('%s' %self.progname)

        self.gridlayout = QGridLayout(self)

        la = QLabel()
        la.setText(self.pinfo)
        self.gridlayout.addWidget(la, 0, 0, 1, 5, Qt.AlignCenter | Qt.AlignJustify)

        self.widgets = OrderedDict()
        
        self.beginners = []
        self.intermediates = []
        self.experts = []
        for self.cellrow, self.name in enumerate(self.parameters):
            if self.level[self.name] in ['beginner', 'intermediate', 'expert']:
                self.label = QLabel()
                # prepare button
                self.button = QPushButton()
                self.openbutton = QPushButton()
                if type(self.parameters[self.name]) is bool:
                    self.insertCheckBox()
                    
                elif type(self.parameters[self.name]) in [int, float] or type(self.parameters[self.name]) is tuple:
                    self.prepareSpinBox()
                    if type(self.parameters[self.name]) in [int, float]:
                        self.insertOneSpinBox()
                    elif type(self.parameters[self.name]) is tuple:
                        self.insertDoubleSpinBox()
                elif type(self.parameters[self.name]) == str:
                    if self.properties[self.name].__str__().startswith('file'):
                        self.insertLineEdit()
                        self.setLineEditAccordingToFileProperties()
                    elif self.properties[self.name].__str__().startswith('choice'):
                        self.insertComboBox()
                self.gridlayout.addWidget(self.label, self.cellrow + 1, 1)
                self.connectCheckBoxToEnableOther()
                self.widgets[self.name + 'label']=self.label
            
        # PushButton Start
        self.pbstart = QPushButton() 
        self.pbstart.setObjectName('Startbutton')
        self.pbstart.setText('OK')
        self.pbstart.setToolTip('Start %s' %self.progname)
        self.gridlayout.addWidget(self.pbstart, self.cellrow + 2, 4)

        # PushButton Stop
        self.pbstop = QPushButton() 
        self.pbstop.setObjectName('Stopbutton')
        self.pbstop.setText('Cancel')
        self.pbstop.setToolTip('Cancel %s' %self.progname)
        self.pbstop.setEnabled(False)
        self.gridlayout.addWidget(self.pbstop, self.cellrow + 2, 3)

        self.gridlayout.setColumnStretch(0, 0)

        # a PushButton to enter the defaults
        self.pbdef = QPushButton()
        self.pbdef.setObjectName('Defaultbutton')
        self.pbdef.setText('Defaults')
        self.pbdef.setFixedWidth(80)
        self.pbdef.setToolTip('Default values are filled from %s' %self.progname)
        self.gridlayout.addWidget(self.pbdef, self.cellrow + 2, 0)
        self.gridlayout.setColumnStretch(0, 0)

        # Progressbar
        self.progressbar = QProgressBar(self)
        self.gridlayout.addWidget(self.progressbar, self.cellrow + 2, 1, 1, 1)

        # Statusbar
        self.statusBar = QStatusBar()
        self.statusBar.showMessage('Status: Ready')
        self.gridlayout.addWidget(self.statusBar, self.cellrow + 3, 1, 1, 3)

        # additional textbrowser
        self.logFrame = QFrame()
        self.logFrame.setFrameStyle(QFrame.StyledPanel|QFrame.Sunken)

        self.loglayout = QVBoxLayout()
        self.logFrame.setMinimumWidth(500)
        self.logFrame.setLayout(self.loglayout)
        self.textBrowser = QTextBrowser(self)
        self.textBrowser.setFontPointSize(QFont().pointSize()-2)
        self.loglayout.addWidget(self.textBrowser)
        self.textBrowser.clear()
#        self.gridlayout.addWidget(self.logFrame, 0, 5, self.cellrow + 2, 5)
        self.gridlayout.addWidget(self.logFrame, self.cellrow + 4, 1, 6, 4)
        self.logFrame.hide()

        # CheckBox expand dialog
        self.logButton = QCheckBox()
        self.logButton.setCheckable(True)
        self.logButton.setText('Log')
        self.logButton.setToolTip('Log will display logfile in console below')
        self.gridlayout.addWidget(self.logButton, self.cellrow + 2, 5)
        
        self.setupAdditionalParameters()
            

    def activateAdditionalOptions(self):
        
        sel_keys = [ self.additional_keys[each_index.row()] for each_index in self.listoptions.selection.selectedIndexes() ]
        added_relatives = [self.relatives[each_key] for each_key in sel_keys if each_key in self.relatives.keys()]
        
        sel_keys += added_relatives
        self.active_parameter_keys = self.minimal_param_keys + sel_keys
        
        widget_keys = [each_widg for each_widg in self.widgets if each_widg not in self.minimal_widg_keys]
        [self.widgets[each_widg].setVisible(False) for each_widg in widget_keys]
        
        for each_key in sel_keys:
            widget_keys = [each_widg for each_widg in self.widgets if each_widg.startswith(each_key)]
            [self.widgets[each_widg].setVisible(True) for each_widg in widget_keys]


    def updateMinimalKeys(self):
        self.minimal_widg_keys = []
        self.minimal_param_keys = []
        for each_key in self.active_parameter_keys:
            self.minimal_param_keys.append(each_key)
            attached_keys = [each_widg for each_widg in self.widgets if each_widg.startswith(each_key)]
            self.minimal_widg_keys += attached_keys


    def setupAdditionalParameters(self):
        self.levels = ['beginner', 'intermediate', 'expert']
        self.levelchoice = QComboBox()
        self.levelchoice.addItems([each_level.title() for each_level in self.levels])
        self.levelchoice.setToolTip('Change the level of input parameters from beginner to expert.')
        self.connect(self.levelchoice, SIGNAL('currentIndexChanged(int)'), self.hideEntriesBasedOnLevel)
        self.gridlayout.addWidget(self.levelchoice, 0, 7, 1, 1)
        
        self.listlabel = QLabel()
        self.listlabel.setText('Additional parameters (level)')
        self.gridlayout.addWidget(self.listlabel, 0, 6, 1, 1)
        
        self.listoptions = QListWidget()
        self.listoptions.setSelectionMode(QAbstractItemView.MultiSelection)
        self.listoptions.selection = self.listoptions.selectionModel()
        self.listoptions.setToolTip('Choose additional parameters.')
        self.connect(self.listoptions, SIGNAL('itemClicked(QListWidgetItem*)'), self.activateAdditionalOptions)
        self.gridlayout.addWidget(self.listoptions, 1, 6, self.cellrow - 1, 2)
        self.additional_keys = [each_name for each_name in self.parameters if self.level[each_name] != 'beginner']
        self.listoptions.addItems(self.additional_keys)
        
        self.active_parameter_keys = [each_key for each_key in self.level if self.level[each_key] == 'beginner']
        self.updateMinimalKeys()

        
class GUI(QWidget, GuiWindow):
    """
    * Class to receive input parameters from GUI

    #. Input = default parameter dictionary
    #. Output = updated parameter dictionary
    """
    def __init__(self, parset, parent = None):
        QWidget.__init__(self, parent)

        self.parset = parset
        self.parameters = parset.parameters
        self.default_params = parset.parameters.copy()
        self.pinfo = parset.proginfo
        self.hints = parset.hints
        self.properties = parset.properties
        self.program_states = parset.program_states
        self.relatives = parset.relatives
        self.level = parset.level

        self.progname = parset.progname

        self.getfilemapper = QSignalMapper(self)
        self.getfilesmapper = QSignalMapper(self)
        self.savefilemapper = QSignalMapper(self)
        self.getdirmapper = QSignalMapper(self)
        self.openbuttonmapper = QSignalMapper(self)
        self.readParamsMapper = QSignalMapper(self)

        # inherit body of layout
        self.setupUi(self)

        # the actual barUpdater thread
        self.lineUpdater = Updater(self.default_params, self.properties, self)
        self.lineUpdater.updatedDirListSignal.connect(partial(self.updateLineEditCompleter))
        # once autocompleter started it is then updated every time mouse is moved
        self.lineUpdater.start()
        self.setMouseTracking(True)

        # the actual barUpdater thread
        self.barUpdater = ProgressBarUpdater(self.parameters, self.program_states, self.progname, self)

        self.connect(self.getfilemapper, SIGNAL('mapped(const QString &)'), self.getFile)
        self.connect(self.getfilesmapper, SIGNAL('mapped(const QString &)'), self.getFiles)
        self.connect(self.savefilemapper, SIGNAL('mapped(const QString &)'), self.saveFile)
        self.connect(self.getdirmapper, SIGNAL('mapped(const QString &)'), self.getDir)
        self.connect(self.openbuttonmapper, SIGNAL('mapped(const QString &)'), self.openDemandedOutput)

        # PushButton Start connect
        self.connect(self.pbstart, SIGNAL('clicked()'), self.readParametersFromGui )

        # PushButton Stop connect
        self.connect(self.pbstop, SIGNAL('clicked()'), self.startOver)

        # a PushButton to enter the defaults
        self.connect(self.pbdef, SIGNAL('clicked()'), self.enterDefaults)

        # receive progressbar
        self.barUpdater.progress.connect(self.progressbar.setValue)

        # CheckBox expand dialog
        self.connect(self.logButton, SIGNAL('toggled(bool)'), self.logFrame, SLOT('setVisible(bool)'))

        # update log continuously
        self.barUpdater.log.connect(partial(self.textBrowser.append))
        self.barUpdater.err.connect(partial(self.statusBar.showMessage))
        
        # switch the enabled states of the actions according to whether the
        # barUpdater is running or not
        self.barUpdater.finished.connect(partial(self.jobDone))

    def startOver(self):
        self.barUpdater.stop()
        self.guiJob.stop()
        self.startOverSignal = True
        

    def getFileFromDialog(self, value):
        rkey = list(self.parameters.keys())[int(value)]
        fname = QFileDialog.getOpenFileName(self, 'Choose file', '.', 
            'Files ({0})'.format('*.' + ' *.'.join(self.properties[rkey].ext)))
        return fname

    def getFile(self, value):	
        fname = self.getFileFromDialog(value) 
        
        if fname != '':
            self.setTextInWidget(fname, value)


    def getFilesFromDialog(self, value, dir=os.curdir):
        rkey = list(self.parameters.keys())[int(value)]
        
        fname = QFileDialog.getOpenFileNames(self, 'Choose files', '{0}'.format(dir), 'Files ({0})'.\
        format('*.' + ' *.'.join(self.properties[rkey].ext)))
        
        return fname

    def getFiles(self, value):	
        fname = self.getFilesFromDialog(value) 
        
        if fname != []:
            self.setTextInWidget(fname.join(','), value)

    def saveFile(self, value):
        rkey = list(self.parameters.keys())[int(value)]
        
        fname = str(QFileDialog.getSaveFileName(self , 'Save file', '.', 'Files ({0})'.\
        format('*.' + ' *.'.join(self.properties[rkey].ext)))) 
        
        if fname != '':
            self.setTextInWidget(fname, value)

    def getDir(self, value):
        fname = str(QFileDialog.getExistingDirectory(self , 'Choose directory', '.')) 
        if fname != '':
            self.setTextInWidget(fname, value)

    def setTextInWidget(self, fname, value):
        self.thisWidget=self.widgets[list(self.parameters.keys())[int(value)]]
        self.thisWidget.setText(fname)


    def enterParametersInGui(self, parameters):
        for name in parameters:
            if name in self.active_parameter_keys:
                value = parameters[name]
            else:
                value = None
                
            if type(value) in [int, float]:
                self.thisWidget = self.widgets[name]
                self.thisWidget.setValue(value)
            elif type(value) is bool:
                if value is True:
                    self.thisWidget = self.widgets[name]
                    self.thisWidget.setCheckState(Qt.Checked)
                elif value is False:
                    self.thisWidget = self.widgets[name]
                    self.thisWidget.setCheckState(Qt.Unchecked)
            elif type(value) is tuple:
                self.thisWidget = self.widgets[name + '1']
                self.thisWidget.setValue(value[0])
                self.thisWidget = self.widgets[name + '2']
                self.thisWidget.setValue(value[1])
            elif type(value) is str:
                self.thisWidget = self.widgets[name]
                if self.properties[name].__str__().startswith('file'):
                    self.thisWidget.setText(value)
                elif self.properties[name].__str__().startswith('choice'):
                    index = self.thisWidget.findText(value.capitalize())
                    self.thisWidget.setCurrentIndex(index)


    def enterDefaults(self):
        self.enterParametersInGui(self.default_params)
        self.pbstart.setToolTip('Start {0} now.'.format(self.progname))

    def readParametersFromGui(self, event=None):
        self.scanParams()
        if self.warn is True:
            QMessageBox.warning(self, 'WARNING: incomplete set of input parameters',
                """<p>WARNING: Incomplete set of input parameters. Please enter requested \
                parameters in \'{0}\'.</br></p>""".format(self.last_name))
        else:
            self.jobStart()

    def scanParams(self):
        self.warn = False
        for self.last_name in self.parameters:
            if self.last_name in self.active_parameter_keys:
                if type(self.default_params[self.last_name]) is tuple:
                    self.thisWidget=self.widgets[self.last_name + '1']
                else:
                    self.thisWidget=self.widgets[self.last_name]
                
                if type(self.default_params[self.last_name]) is str and \
                not self.properties[self.last_name].__str__().startswith('choice'):
                    if not self.thisWidget.isHidden() and self.thisWidget.isEnabled():
                        if str(self.thisWidget.text()) is '':
                            self.warn = True
                            break


    def hideEntriesBasedOnLevel(self, intLevel):
        currentLevel = self.levels[intLevel]
        if currentLevel in ['expert']:
            self.active_parameter_keys = [each_key for each_key in self.level
                                          if self.level[each_key] in ['beginner', 'intermediate', 'expert']]

            self.additional_keys = []
            self.listoptions.setVisible(False)
            for every_item in self.beginners + self.intermediates + self.experts:
                every_item.show()
        else:
            for every_item in self.beginners + self.intermediates + self.experts:
                every_item.hide()
        
        if currentLevel in ['intermediate']:
            self.active_parameter_keys = [each_key for each_key in self.level 
                                          if self.level[each_key] in ['beginner', 'intermediate']]

            self.additional_keys = [each_name for each_name in self.parameters if self.level[each_name] in ['expert']]
            for every_item in self.beginners + self.intermediates:
                every_item.show()
        elif currentLevel in ['beginner']:
            self.active_parameter_keys = [each_key for each_key in self.level if self.level[each_key] in ['beginner']]
            self.additional_keys = [each_name for each_name in self.parameters 
                                    if self.level[each_name] in ['intermediate', 'expert']]

            for every_item in self.beginners:
                every_item.show()
                
        if currentLevel in ['beginner', 'intermediate']:
            self.listoptions.setVisible(True)
            self.listoptions.clear()
            self.listoptions.addItems(self.additional_keys)

        self.updateMinimalKeys()
    
    def readParams(self):
        for name in self.default_params:
            if name in self.active_parameter_keys:
                if type(self.default_params[name]) is bool:
                    self.thisWidget=self.widgets[name]
                    if self.thisWidget.isChecked():
                        self.parameters[name]=True
                    else:
                        self.parameters[name]=False
                elif type(self.default_params[name]) is int: 
                    self.thisWidget=self.widgets[name]
                    self.parameters[name]=int(self.thisWidget.value())
                elif type(self.default_params[name]) is float: 
                    self.thisWidget=self.widgets[name]
                    self.parameters[name]=float(self.thisWidget.value())
                elif type(self.default_params[name]) is tuple:
                    self.thisWidget=self.widgets[name + '1']
                    if type(self.default_params[name][0]) is int: 
                        first=int(self.thisWidget.value())
                    elif type(self.default_params[name][0]) is float: 
                        first=float(self.thisWidget.value())
                    self.thisWidget=self.widgets[name + '2']
                    if type(self.default_params[name][0]) is int: 
                        second=int(self.thisWidget.value())
                    elif type(self.default_params[name][0]) is float: 
                        second=float(self.thisWidget.value())
                    self.parameters[name]=tuple((first, second))
                elif type(self.default_params[name]) is str:
                    self.thisWidget=self.widgets[name]
                    if self.properties[name].__str__().startswith('file'):
                        self.parameters[name] = str(str(self.thisWidget.text()))
                    elif self.properties[name].__str__().startswith('choice'):
                        self.parameters[name] = str(str(self.thisWidget.currentText()))
#                if self.thisWidget.isHidden() and self.level[name] not in ['beginner']:
#                    self.parameters[name]=None
            else:
                self.parameters[name]=None
        

    def jobSetup(self):
        self.prgparfile = Features().write_parfile(self.parameters, self.progname)
        self.directory = NameGenerator().make_directory(self.progname)
        self.logfile = 'report' + os.extsep + 'log'#NameGenerator().make_logfile(self.progname)#
        self.dirprogname = '--f {0} --l {1} --d {2}'.format(self.prgparfile, self.logfile, self.directory)
        self.barUpdater.logfile = self.directory + os.sep + self.logfile
        
        
    def jobStart(self):
        self.statusBar.showMessage('Status: Initialized.')
        self.pbstart.setEnabled(False)
        self.pbstop.setEnabled(True)
        self.pbstop.setText('Cancel')
        self.readParams()
        self.startOverSignal = False
        self.enableOpenButton(False)
        self.jobSetup()
        self.guiJob = LauncherGui()
        self.guiJob.finished.connect(partial(self.jobDone))
        self.guiJob.pass_program_command_line(self.progname, self.dirprogname.split())
#        self.jobSetup()
#        self.guiJob = LauncherGui(self.progname, self.dirprogname.split())
#        self.guiJob.finished.connect(partial(self.jobDone))
        self.guiJob.start()
        self.barUpdater.start()

    def enableOpenButton(self, TrueOrFalse):
        for self.name in self.parameters:
            if type(self.parameters[self.name]) == str:
                if self.properties[self.name].__str__().startswith('file') and \
                                self.properties[self.name].ftype == 'saveFile':
                    self.widgets[self.name + 'openbutton'].setEnabled(TrueOrFalse)
                    self.widgets[self.name + 'openbutton'].setToolTip('Open %s' % self.parameters[self.name])

    def jobDone(self):
        self.pbstop.setEnabled(False)
        self.pbstart.setEnabled(True)
#        self.guiJob.finished()
        
        if os.path.exists(self.prgparfile):
            os.rename(self.prgparfile, self.directory + os.sep + 'parameters.par')
        if self.startOverSignal is False:
            self.enableOpenButton(True)
                    
    def openDemandedOutput(self, value):
        rkey = list(self.parameters.keys())[int(value)]
        outfile = self.directory + os.sep + os.path.basename(self.parameters[rkey])
        self.external_program = ExtLauncher()
        if os.path.exists(outfile):
            outfiles = [outfile]
        else:
            outfiles = self.getFilesFromDialog(value, self.directory)
        if outfiles != []:
            for each_file in outfiles:
                self.external_program.qlaunch_open_file(each_file)

    def mouseMoveEvent(self, event):
#        print 'Mouse Pointer is currently hovering at: ', event.pos()
        self.lineUpdater.start()

    def updateLineEditCompleter(self, completeDict):
        for name in self.default_params:
            if type(self.default_params[name]) is str:
                if self.properties[name].__str__().startswith('file'):
                    self.lineEditCompleter = QCompleter(completeDict[name])
                    self.lineEditCompleter.setCompletionMode(QCompleter.InlineCompletion)
                    self.lineEditCompleter.setCaseSensitivity(Qt.CaseInsensitive)
                
                    self.widgets[name].setCompleter(self.lineEditCompleter)

class Updater(QThread):
    """
    Class of function to perform update tasks
    """
    updatedDirListSignal = pyqtSignal(dict)
    def __init__(self, defdict=None, rangedict=None, parent=None):
        QThread.__init__(self, parent)

        self.default_params = defdict
        self.properties = rangedict

        self._run_semaphore = QSemaphore(1)

    def run(self):
        
        # Note: This is never called directly. It is called by Qt once the
        # thread environment has been set up.
        self.updateFileAutoCompleter()

    def updateFileAutoCompleter(self):
        completeDict = {}
        QApplication.processEvents()
        for name in self.default_params:
            if type(self.default_params[name]) == str:
                if self.properties[name].__str__().startswith('file'):
                    # set up extension-specific completer 
                    completeDirList = []
                    for ext in self.properties[name].ext:
                        completeDirList += glob('*{0}{1}'.format(os.extsep, ext))
             
                    completeDict[name] = completeDirList
        self.updatedDirListSignal.emit(completeDict)
        sleep(random())

    def __del__(self):
        self.wait()


class QTabWidgetCloseable(QTabWidget):
    def __init__(self, parent = None):
        QTabWidget.__init__(self, parent)
        
        self.setTabsClosable(True)
        self.connect(self, SIGNAL('tabCloseRequested(int)'), self.closeTab)
        
        
    def closeTab(self, tab_id):
        if tab_id != 0:
            self.removeTab(tab_id)
            
        


#!/usr/bin/env python
"""
Test module to check particleclass
"""
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.particle2d.particleclass import ParticleClass, ParticleClassPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment2d.test_segmentclass import TestSegmentClassClear, \
    HelixClassPreparation
import os


class TestParticleClassPreparation(object):
    def setup_helix_or_particle_dimensions(self):
        self.feature_set.parameters['Estimated inner and outer particle diameter in Angstrom'] = ((200, 700))

class TestParticleClass(TestParticleClassPreparation, ParticleClass, TestSegmentClassClear, HelixClassPreparation):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = ParticleClassPar()
        self.setup_common_non_mpi_parameters()
        
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())

        super(TestParticleClass, self).__init__(self.feature_set)

    def do_test_case_pc1(self):
        self.classify()

    def teardown(self):
        self.teardown_segmentclass()
        self.testingdir.remove()


class TestParticleClassMore(TestParticleClass):
    def do_test_case_sc2(self):
        self.feature_set.parameters['Reference option']=True

        super(TestParticleClass, self).__init__(self.feature_set)
        self.classify()

class TestParticleClassEndToEnd(TestParticleClass):
    def do_end_to_end_test_pc_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_pc_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)

class TestParticleClassMpi(TestParticleClass):
    def do_end_to_end_test_pc_inputfile_MPI(self):

        self.feature_set.parameters['MPI option']=True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)


def main():
    tsce = TestParticleClass()
    tsce.setup()
    tsce.do_test_case_pc1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 12-Jun-2013
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from EMAN2 import EMData, EMNumPy, Util, periodogram
from collections import namedtuple
from fundamentals import mirror
from morphology import binarize
from scipy import special
from spring.particle2d.particlealign2d import ParticleAlign2d
from spring.micprgs.scansplit import Micrograph
from spring.segment2d.segmentexam import SegmentExam
from spring.segment3d.segclassreconstruct_assist import SegClassReconstructAssist
from statistics import ccc
from tabulate import tabulate
from utilities import model_circle, model_blank
import numpy as np
import os


class SegClassReconstructCompareMontage(SegClassReconstructAssist):  
    def normalize_power_spectrum(self, power):
        stat = Micrograph().get_statistics_from_image(power)
        freq, gray_bins = np.histogram(np.copy(EMNumPy.em2numpy(power)), 65535)
        most_frequent = gray_bins[np.argmax(freq)]
        
        power = (power - most_frequent) / stat.sigma
         
        return power
    

    def window_left_half_of_spectrum(self, segment_size, thresholded_class_periodogram):
        if (segment_size) % 2 == 0:
            half_panel_width = segment_size / 2 + 1
        else:
            half_panel_width = segment_size / 2
        offset = int(-segment_size / 4) + 1
        if (half_panel_width + offset) > segment_size / 4:
            offset += 1
        windowed_class_periodogram = Util.window(thresholded_class_periodogram, half_panel_width, segment_size, 1, 
            offset, 0, 0)
        
        montage_width = half_panel_width * 2
        
        return windowed_class_periodogram, montage_width
    

    def prepare_class_avg_periodogram_for_montage(self, class_avg_periodogram, segment_size):
        thresholded_class_periodogram = \
        Micrograph().adjust_gray_values_for_print_and_optimal_display(class_avg_periodogram)
        
        thresholded_class_periodogram = self.normalize_power_spectrum(thresholded_class_periodogram)
        
        windowed_class_periodogram, montage_width = \
        self.window_left_half_of_spectrum(segment_size, thresholded_class_periodogram)
        
        windowed_class_periodogram = Util.pad(windowed_class_periodogram, montage_width, segment_size, 1, 
        -montage_width / 4, 0, 0, '0')
        
        return windowed_class_periodogram


    def montage_class_avg_vs_reprojection_periodogram(self, segment_size, windowed_class_periodogram,
    reprojection_periodogram):
        thresholded_periodogram = \
        Micrograph().adjust_gray_values_for_print_and_optimal_display(reprojection_periodogram)
        
        thresholded_periodogram = self.normalize_power_spectrum(thresholded_periodogram)
        
        windowed_helix_periodogram, montage_width = \
        self.window_left_half_of_spectrum(segment_size, thresholded_periodogram)
        
        windowed_helix_periodogram = mirror(windowed_helix_periodogram)
        
        windowed_helix_periodogram = Util.pad((windowed_helix_periodogram), montage_width, segment_size, 1, 
        montage_width / 4, 0, 0, '0')
        
        class_vs_helix_periodogram_montage = (windowed_class_periodogram + windowed_helix_periodogram) * (-1)
        
        return class_vs_helix_periodogram_montage
    

    def montage_exp_vs_sim_power_spectrum(self, power_spectrum, ideal_power_img):
        """
        >>> from utilities import model_gauss_noise
        >>> power_size = 25
        >>> zero = model_gauss_noise(1, power_size, power_size)
        >>> one = model_gauss_noise(10, power_size, power_size)
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> montage = SegClassReconstruct().montage_exp_vs_sim_power_spectrum(one, zero)
        """
        power_size = power_spectrum.get_xsize()
        prepared_power = self.prepare_class_avg_periodogram_for_montage(power_spectrum, power_size)
        
        montage_side_by_side = self.montage_class_avg_vs_reprojection_periodogram(power_size, prepared_power,
        ideal_power_img)
 
        return montage_side_by_side
 

class SegClassReconstructCompare(SegClassReconstructCompareMontage):  
    def project_through_reference_using_parameters_and_log(self, projection_parameters, alignment_size, prj_ids,
    projection_stack, reference_volume):
        self.project_locally(reference_volume, projection_parameters, projection_stack)
        proj_loginfo = []
        img = EMData()
        img.read_image(projection_stack)
        img_width = img.get_xsize() 
        for each_prj_id, each_projection_parameter in enumerate(projection_parameters):
            each_total_prj_id = prj_ids[each_prj_id]
            proj_loginfo += [[each_total_prj_id, each_prj_id] + each_projection_parameter]
            img.read_image(projection_stack, each_prj_id)
#             img = Util.window(img, img_width, alignment_size, 1, 0, 0, 0)
            if img_width < alignment_size:
                img = Util.pad(img, alignment_size, alignment_size, 1, 0, 0, 0, 'avg')
            if img_width > alignment_size:
                img = Util.window(img, alignment_size, alignment_size, 1, 0, 0, 0)
                
            img.write_image(projection_stack, each_prj_id)
        
        msg = tabulate(proj_loginfo, ['stack_id', 'local_id', 'phi', 'theta', 'psi', 'x-shift', 'y-shift'])
        self.log.tlog('The volume was projected according to the following parameters:\n{0}'.format(msg))
        
        return projection_stack


    def make_rec_stack_info(self):
        return namedtuple('stack_info', 'file_name reference_vol alignment_size')


    def get_helical_error_string(self):
        return 'helical_ccc_error mean_helical_ccc phase_residual ' + \
        'amp_correlation, amp_corr_quarter_nyquist, amp_corr_half_nyquist, amp_corr_3quarter_nyquist ' + \
        'segment_count asym_unit_count out_of_plane_dev'
    

    def make_helical_error_tuple(self):
        err_tuple = namedtuple('errors_helical', self.get_helical_error_string())
        return err_tuple
        

    def get_quarter_half_3quarter_nyquist_average_from_amp_correlation(self, amp_correlation):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> SegClassReconstruct().get_quarter_half_3quarter_nyquist_average_from_amp_correlation(range(10))
        [0.5, 2.0, 3.0, 4.5]
        """
        vals = [np.mean(amp_correlation[:int(each_ratio*len(amp_correlation))]) for each_ratio in [0.25, 0.5, 0.75, 1.0]]
        
        return vals
    
        
    def compute_amplitude_correlation_between_sim_and_exp_power_spectrum(self, sim_power, exp_power):
        power_size = sim_power.get_xsize()
        max_radius = power_size / 2
        
        amp_cc = []
        for each_radius in range(max_radius):
            outer_circle = model_circle(each_radius + 1, power_size, power_size)
            inner_circle = model_circle(each_radius, power_size, power_size)
            circle = outer_circle - inner_circle
            
            amp_cc.append(ccc(sim_power, exp_power, circle))
            
        return amp_cc
    
        
    def pad_mask_and_compute_power(self, prj, pad_size, rect_mask=None):
        if rect_mask is not None:
            prj *= rect_mask
        pad_prj = Util.pad(prj, pad_size * prj.get_xsize(), pad_size * prj.get_ysize(), 1, 0, 0, 0, '0')
        sim_power = periodogram(pad_prj)

        return sim_power


    def determine_mean_ccc_deviation_within_symmetry_related_views(self, rec_stack, Euler_angles_rec, helixwidthpix,
    measure_phase_residual=False, measure_amp_corr=False, keep_central=False):
        
        reference_volume = EMData()
        reference_volume.read_image(rec_stack.reference_vol)
        
        prj_ids = np.arange(len(Euler_angles_rec))
        sym_ids = Euler_angles_rec[:,0]
        stack_ids = Euler_angles_rec[:,1]
        
        phis = Euler_angles_rec[:,2]
        thetas = Euler_angles_rec[:,3]
        psis = Euler_angles_rec[:,4]
        xx = Euler_angles_rec[:,5]
        yy = Euler_angles_rec[:,6]
        
        prj_Eulers = [[phi, theta, psi, 0.0, 0.0] for (phi, theta, psi) in zip(phis, thetas, psis)]

        projection_stack = os.path.join(os.path.dirname(rec_stack.file_name), 'rec_stack_reproj.hdf')
        
        projection_stack = self.project_through_reference_using_parameters_and_log(prj_Eulers, rec_stack.alignment_size,
        prj_ids, projection_stack, reference_volume)
        
        rect_mask = SegmentExam().make_smooth_rectangular_mask(helixwidthpix, rec_stack.alignment_size, 
        rec_stack.alignment_size, 0.15)

        circle = ParticleAlign2d().make_smooth_circular_mask(int(0.70 * rec_stack.alignment_size), 0,
        rec_stack.alignment_size, 0.15)
 
        rect_mask *= circle
        
        prj = EMData()
        rec_img = EMData()
        
        stdevs = []
        cccs = []
        phase_resid = []
        pad_size = 2
        amp_corrs = np.array([0.0] * (rec_stack.alignment_size * pad_size / 2))
        amp_count = 0
        
        central_prj = []
        central_rec_img = []
        for each_stack_id in np.unique(stack_ids):
            this_sym_ids = np.int64(sym_ids[stack_ids == each_stack_id])
            if np.isscalar(this_sym_ids):
                this_sym_ids = [this_sym_ids]
            stack_id_cccs = []
            for each_prj_id, each_sym_id in enumerate(this_sym_ids):
                rec_img.read_image(rec_stack.file_name, int(each_sym_id))
                prj.read_image(projection_stack, each_prj_id)
                stack_id_cccs.append(ccc(prj, rec_img, rect_mask))
                if measure_phase_residual:
                    phase_resid.append(prj.cmp('phase', rec_img))
                if measure_amp_corr:
                    sim_power = self.pad_mask_and_compute_power(prj, pad_size, rect_mask)
                    exp_power = self.pad_mask_and_compute_power(rec_img, pad_size, rect_mask)

                    pad_img = Util.pad(rec_img * rect_mask, pad_size * rec_stack.alignment_size, 
                    pad_size * rec_stack.alignment_size, 1, 0, 0, 0, 'average')

                    exp_power = periodogram(pad_img)
                    amp_res = self.compute_amplitude_correlation_between_sim_and_exp_power_spectrum(sim_power, exp_power)
                    amp_corrs += amp_res
                    amp_count += 1
#                rec_img *= rect_mask
#                rec_img.append_image('mytest.hdf')
#                prj.append_image('mytest.hdf')
            stdevs.append(np.std(stack_id_cccs))
            cccs += stack_id_cccs
            if keep_central:
                dist = np.sqrt(xx[this_sym_ids] **2 + yy[this_sym_ids] **2)
                central_id = int(this_sym_ids[np.argmin(dist)])
                prj.read_image(projection_stack, central_id)
                central_prj.append(prj.copy())
                rec_img.read_image(rec_stack.file_name, central_id)
                central_rec_img.append(rec_img.copy())
            
        if measure_amp_corr and amp_count !=0:
            amp_corrs /= float(amp_count)

        if measure_phase_residual:
            phase_resid_avg = np.mean(phase_resid)
        else:
            phase_resid_avg = None

        segment_count = len(np.unique(stack_ids))
        asym_unit_count = len(sym_ids)
        quarter, half, three_quarter, full = \
        self.get_quarter_half_3quarter_nyquist_average_from_amp_correlation(amp_corrs)

        err_tuple = self.make_helical_error_tuple()

        mean_error = err_tuple(np.mean(stdevs), np.mean(cccs), phase_resid_avg, full, quarter, half, three_quarter, 
        segment_count, asym_unit_count, None) 
        
        os.remove(projection_stack)

        return mean_error, central_prj, central_rec_img
    
    
class SegClassReconstructComparePredictLayerLines(SegClassReconstructCompare):

    def get_list_of_bessel_order_maxima(self, order_count):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> SegClassReconstruct().get_list_of_bessel_order_maxima(10)
        array([  0. ,   1.8,   3.1,   4.2,   5.3,   6.4,   7.5,   8.6,   9.6,  10.7])
        """
        arr = np.arange(0, 25, 0.1)
        primarymax = np.array([arr[np.argmax(special.jv(each_order, arr))] for each_order in range(order_count)])
        
        return primarymax
    

    def adjust_bessel_order_if_out_of_plane_not_zero(self, out_of_plane_tilt, layer_line_position,
    assigned_bessel_order, helix_radius, bessel_maxima):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> bes = s.get_list_of_bessel_order_maxima(20)
        >>> s.adjust_bessel_order_if_out_of_plane_not_zero(0, 0.1, 3, 180.0, bes) 
        3
        >>> s.adjust_bessel_order_if_out_of_plane_not_zero(3, 0.1, 3, 180.0, bes)
        0
        """
        if out_of_plane_tilt != 0:
            meridional_dist = np.tan(np.deg2rad(out_of_plane_tilt)) * layer_line_position
            pos_prim_point = 2 * np.pi * meridional_dist * helix_radius
            shifted_maxs = bessel_maxima - pos_prim_point
            adjusted_bessel_order = np.argmin(np.abs(shifted_maxs[abs(assigned_bessel_order)] - bessel_maxima))
            if assigned_bessel_order < 0:
                adjusted_bessel_order = -adjusted_bessel_order
        else:
            adjusted_bessel_order = assigned_bessel_order

        return adjusted_bessel_order


    def adjust_reciprocal_layer_line_pitches_by_out_of_plane_angle(self, pitches, out_of_plane_tilt=0):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> s.adjust_reciprocal_layer_line_pitches_by_out_of_plane_angle(0.1, 0)
        0.10000000000000001
        >>> s.adjust_reciprocal_layer_line_pitches_by_out_of_plane_angle(0.1, 12)
        0.10223405948650292
        """
        tilt_corrected_pitches = pitches / np.cos(np.deg2rad(out_of_plane_tilt)) 

        return tilt_corrected_pitches


    def generate_layerline_bessel_pairs_from_rise_and_rotation(self, rise_rotation_pair, rot_sym, width_of_helix,
    pixelsize, low_resolution_cutoff, high_resolution_cutoff, out_of_plane_tilt=0.0):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> s = SegClassReconstruct()
        >>> sym = (1.408, 22.03)
        >>> s.generate_layerline_bessel_pairs_from_rise_and_rotation(sym, 1, 180.0, 5.0, 300, 10) #doctest: +NORMALIZE_WHITESPACE
        [(0.086926286509040343, 2), (0.072087658592848908, 18), 
        (0.058298839853086926, -15), (0.043461254291798861, 1), 
        (0.028626227349497609, 17), (0.01483591478250549, -16)]
        >>> s.generate_layerline_bessel_pairs_from_rise_and_rotation(sym, 1, 180.0, 5.0, 300, 10, 3) #doctest: +NORMALIZE_WHITESPACE
        [(0.087045579450445165, 0), (0.072186587802618302, 16), 
        (0.058378846032642748, -13), (0.043520898170190836, 0), 
        (0.028665512438036268, 16), (0.014856274790782758, -16)]
        >>> s.generate_layerline_bessel_pairs_from_rise_and_rotation(sym, 2, 180.0, 5.0, 300, 10, 3) #doctest: +NORMALIZE_WHITESPACE
        [(0.087045579450445165, 0), (0.072186587802618302, 16), 
        (0.014856274790782758, -16)]
        """
        helical_rise = rise_rotation_pair[0]
        helical_rotation = rise_rotation_pair[1]
        pitch_of_helix = helical_rise * 360 / helical_rotation
        
        helix_radius = width_of_helix / 2.0
        maximum_n = int((np.pi * helix_radius)/pixelsize) + 2 
        pitch_grid = np.zeros((maximum_n, maximum_n))
        bessel_grid = np.zeros((maximum_n, maximum_n))
        
        n_combinations = m_combinations = list(range(-maximum_n / 2, maximum_n / 2, 1))
        for n_index, each_n in enumerate(n_combinations):
            n_is_multiple_of_rotational_symmetry = each_n % rot_sym
            if n_is_multiple_of_rotational_symmetry == 0:
                for m_index, each_m in enumerate(m_combinations):
                    layer_line_position = each_n / pitch_of_helix + each_m / helical_rise
                    if layer_line_position == 0:
                        pass
                    else:
                        pitch_grid[n_index][m_index]=1 / layer_line_position
                        bessel_grid[n_index][m_index]=each_n
            
#        img = plt.imshow(pitch_grid, cmap=cm.jet, interpolation='nearest')
#        plt.savefig('test.pdf', dpi=600)
                
        pitch_sequence = pitch_grid.ravel()
        
        bessel_maxima = self.get_list_of_bessel_order_maxima(1000)
        unique_pitches = np.unique(np.round(np.abs(pitch_sequence), 3))
        pitches = []
        orders = []
        for each_unique_pitch in unique_pitches:
            if high_resolution_cutoff < each_unique_pitch < low_resolution_cutoff:
                n_m_indices_of_pitch = np.argwhere(np.round(pitch_grid, 3) == each_unique_pitch)
                
                bessel_orders = np.zeros(len(n_m_indices_of_pitch))
                for each_index, each_n_m_index in enumerate(n_m_indices_of_pitch):
                    each_n_idx, each_m_idx = each_n_m_index
                    bessel_order = bessel_grid[each_n_idx][each_m_idx]
                    bessel_orders[each_index]=bessel_order
                
                if len(n_m_indices_of_pitch) > 0:
                    min_index = np.argmin(np.abs(bessel_orders))
                    assigned_bessel_order = int(bessel_orders[min_index])
                
                    layer_line_position = 1 / each_unique_pitch
                    pitches.append(layer_line_position)
                    
                    adjusted_bessel_order = self.adjust_bessel_order_if_out_of_plane_not_zero(out_of_plane_tilt,
                    layer_line_position, assigned_bessel_order, helix_radius, bessel_maxima)
                    
                    orders.append(adjusted_bessel_order)

        pitches = np.array(pitches)
        
        tilt_corrected_pitches = self.adjust_reciprocal_layer_line_pitches_by_out_of_plane_angle(pitches,
        out_of_plane_tilt)
        
        unique_pitch_bessel_pairs = list(zip(tilt_corrected_pitches, orders))
        
        msg = tabulate(unique_pitch_bessel_pairs, ['layer_line_position', 'bessel_order'])
        self.log.ilog('The following layer_line_position/Bessel pairs have been computed:\n{0}'.\
        format(msg))
        
        return unique_pitch_bessel_pairs
    
            
class SegClassReconstructCompareLayerFilter(SegClassReconstructComparePredictLayerLines):
    def create_single_layer_line(self, linex, linex_fine, bessel_order, layer_line_length):
        """
        >>> from spring.segment3d.segclassreconstruct import SegClassReconstruct
        >>> b_len = np.pi * 180.0 / 1.2
        >>> ll_len = 50
        >>> b_fine = np.linspace(0, b_len, 100 * ll_len)
        >>> b = np.linspace(0, b_len, ll_len)
        >>> s = SegClassReconstruct()
        >>> ll = s.create_single_layer_line(b, b_fine, 4, ll_len)
        >>> np.round(ll, 2)
        array([ 0.  ,  0.26,  0.18,  0.15,  0.13,  0.11,  0.09,  0.08,  0.06,
                0.04,  0.03,  0.01,  0.  ,  0.01,  0.02,  0.03,  0.04,  0.05,
                0.05,  0.06,  0.06,  0.06,  0.05,  0.05,  0.04,  0.03,  0.02,
                0.01,  0.  ,  0.01,  0.02,  0.02,  0.03,  0.04,  0.04,  0.04,
                0.04,  0.04,  0.04,  0.04,  0.03,  0.03,  0.02,  0.01,  0.01,
                0.  ,  0.01,  0.02,  0.02,  0.03])
        """
        bessel_function = np.abs(special.jv(bessel_order, linex_fine))
        layer_line = np.zeros((layer_line_length))
        for each_pixel, each_val in enumerate(layer_line):
            bessel_index_closest = np.argmin(np.abs(linex_fine - linex[each_pixel]))
            layer_line[each_pixel] += abs(bessel_function[bessel_index_closest])
        
        return layer_line
    

    def prepare_ideal_power_spectrum_from_layer_lines(self, layerline_bessel_pairs, width_of_helix, power_size,
    pixelsize, binary=False):
        if type(power_size) != tuple:
            power_width = power_size
        else:
            power_size, power_width = power_size
        bessel_length = np.pi * width_of_helix / pixelsize
        layer_line_length = power_width / 2
        linex = np.linspace(0, bessel_length, layer_line_length)
        linex_fine = np.linspace(0, bessel_length, 100 * layer_line_length)
        
        ideal_layer_line_power = np.zeros((power_size / 2, layer_line_length))
        for each_layer_line_pair in layerline_bessel_pairs:
            layer_line_position_in_pixel = int(round(each_layer_line_pair[0] * pixelsize * power_size))
            if layer_line_position_in_pixel < power_size / 2:
                bessel_order = abs(each_layer_line_pair[1])
                layer_line = self.create_single_layer_line(linex, linex_fine, bessel_order, layer_line_length)
                ideal_layer_line_power[layer_line_position_in_pixel] += layer_line
                    
        ideal_power = np.hstack((np.fliplr(np.flipud(ideal_layer_line_power)), np.flipud(ideal_layer_line_power)))
        upper_half = np.roll(ideal_power, 1, axis=0)
        lower_half = np.flipud(ideal_power)
        ideal_power = np.vstack((upper_half, lower_half))
        ideal_power_img = EMNumPy.numpy2em(np.copy(ideal_power))
        meridian_solid = model_blank(power_width, 1, 1, 1)
        ideal_power_img.set_row(meridian_solid, power_size / 2)
        
        if binary:
            ideal_power_img = binarize(ideal_power_img, 0.000001)
        
        return ideal_power_img, linex_fine
    

    def filter_image_by_fourier_filter_while_padding(self, image, image_size, padsize, ideal_power_img):
        image = Util.pad(image, padsize, padsize, 1, 0, 0, 0, 'average')
        image = image.filter_by_image(ideal_power_img)
        image = Util.window(image, image_size, image_size, 1, 0, 0, 0)
        
        return image
    

class SegClassReconstructCompareVisual(SegClassReconstructCompareLayerFilter):  
    def determine_pixel_width_of_helix_for_display(self, segment_size, helixwidthpix):
        display_width = max(segment_size / 2, int(2.2 * helixwidthpix + 1))
        
        return display_width
     
     
    def prepare_class_avg_for_montage(self, class_avg, segment_size, helixwidthpix):
        display_width = self.determine_pixel_width_of_helix_for_display(segment_size, helixwidthpix)
        
        if display_width < segment_size:
            windowed_class_avg = Util.window(class_avg, display_width, segment_size, 1, 0, 0, 0)
        else:
            windowed_class_avg = class_avg.copy()
            
        windowed_class_avg.process_inplace('normalize')
        
        windowed_class_avg = Util.pad(windowed_class_avg, 2 * display_width, segment_size, 1, -display_width / 2, 0, 0,
        'average')
        
        return windowed_class_avg


    def montage_class_avg_vs_reprojection(self, segment_size, helixwidthpix, windowed_class_avg, symmetry_reprojection):
        thresholded_sym_reprojection = \
        Micrograph().adjust_gray_values_for_print_and_optimal_display(symmetry_reprojection)
        
        display_width = self.determine_pixel_width_of_helix_for_display(segment_size, helixwidthpix)
        if display_width < segment_size:
            windowed_helix = Util.window(thresholded_sym_reprojection, display_width, segment_size, 1, 0, 0, 0)
        else:
            windowed_helix = thresholded_sym_reprojection.copy()
            
        windowed_helix.process_inplace('normalize')
        windowed_helix = Util.pad(windowed_helix, 2 * display_width, segment_size, 1, display_width / 2, 0, 0)
        class_vs_helix_montage = windowed_class_avg + windowed_helix
        
        return class_vs_helix_montage


    def set_header_with_helical_parameters(self, each_rise_rotation_pair, segment, point_symmetry=None):
        decimal_place = 5
        
        rise_rotation_pair = '({0}, {1})'.format(round(each_rise_rotation_pair[0], decimal_place),
        round(each_rise_rotation_pair[1], decimal_place))
        
        segment.set_attr('rise/rotation', rise_rotation_pair)
        
        if each_rise_rotation_pair[1] != 0:
            each_pitch_unit_pair = self.convert_rise_rotation_pair_to_pitch_unit_pair(each_rise_rotation_pair)
        
            pitch_unit_pair = '({0}, {1})'.format(round(each_pitch_unit_pair[0], decimal_place),
            round(each_pitch_unit_pair[1], decimal_place))
        
            segment.set_attr('pitch/unit_number', pitch_unit_pair)
        if point_symmetry is not None:
            segment.set_attr('point_sym', point_symmetry)
        
        return segment
    

    def write_out_side_by_side_display_of_images_and_power_spectra(self, class_avg, reproj, img_id, montage_reprj,
    montage_power, each_rise_rotation_pair):
        
        segment_size = reproj.get_xsize()
        rect_mask = SegmentExam().make_smooth_rectangular_mask(self.helixwidthpix, segment_size, segment_size, 0.15)
        circle = ParticleAlign2d().make_smooth_circular_mask(int(0.70 * segment_size), 0, segment_size, 0.15)
        rect_mask *= circle

        windowed_class_avg = self.prepare_class_avg_for_montage(class_avg * rect_mask, segment_size, self.helixwidthpix)
        
        montage_real = self.montage_class_avg_vs_reprojection(segment_size, self.helixwidthpix, windowed_class_avg,
        reproj * rect_mask)
        
        montage_real = self.set_header_with_helical_parameters(each_rise_rotation_pair, montage_real)
        montage_real.write_image(montage_reprj, img_id)

        pad_size = 3
        sim_power = self.pad_mask_and_compute_power(reproj, pad_size, rect_mask)
        exp_power = self.pad_mask_and_compute_power(class_avg, pad_size, rect_mask)
        montage_fourier = self.montage_exp_vs_sim_power_spectrum(exp_power, sim_power)

        montage_fourier = self.set_header_with_helical_parameters(each_rise_rotation_pair, montage_fourier)
        montage_fourier.write_image(montage_power, img_id)


#!/usr/bin/env python
"""
Test module to check scansplit
"""
from EMAN2 import EMData, EMNumPy, Util
from fundamentals import mirror
from glob import glob
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.scansplit import ScanSplit, ScanSplitPar, Micrograph
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from utilities import model_gauss_noise
import numpy as np
import os
import random
import sys

class ScanPreparation(object):
    def prepare_scan(self):
        """
        * Function to prepare a dummy test scan containing 3 x 2 micrographs
        """
        self.scanned_micrograph = 'test_scanmic.tif'
        self.row_count = 2
        self.column_count = 3
        self.number_of_micrographs = self.row_count*self.column_count
        self.micrograph_width_cm = 8.0
        self.micrograph_height_cm = 9.0
        self.label_width_cm = 3.0
        self.label_height_cm = 2.0
        self.scanner_stepsize_micrometer = 1000
        self.cross_correlation_criterion = 0.5

        self.reference_micrograph = self.make_refmic(self.micrograph_width_cm, self.micrograph_height_cm,
        self.label_width_cm, self.label_height_cm, self.scanner_stepsize_micrometer)
        
        self.reference_micrograph = self.add_noise_to_reference_micrograph(self.reference_micrograph)
        self.change_sigma_of_columns()
        self.pad_reference_micrograph()
        self.stack_reference_micrographs_to_scan()
        self.scan.write_image(self.scanned_micrograph)

#        mirrored_micrograph = mirror(self.scan, axis='y')
#        mirrored_micrograph.write_image(self.scanned_micrograph)
    def add_noise_to_reference_micrograph(self, micrograph):
        self.noise = model_gauss_noise(0.5, micrograph.get_xsize(), micrograph.get_ysize()) + 1
        micrograph += self.noise

        return micrograph

    def change_sigma_of_columns(self):
        column_numbers = [random.randint(int(0.06*self.reference_micrograph.get_xsize()),
        int(0.94*self.reference_micrograph.get_xsize())) for column_number in range(random.randint(2, 10))]
        
        for column_number in column_numbers:
            column = self.reference_micrograph.get_col(column_number)
            column = column*random.random() - random.random()
            self.reference_micrograph.set_col(column, column_number)

    def pad_reference_micrograph(self):
        border_dimension_x = int(1.2*self.micrograph_width_cm*1e+4/(self.scanner_stepsize_micrometer))
        border_dimension_y = int(1.2*self.micrograph_height_cm*1e+4/(self.scanner_stepsize_micrometer))

        self.reference_micrograph = Util.pad(self.reference_micrograph, border_dimension_x, border_dimension_y, 1, 0, 0, 0, '0')
        self.reference_micrograph_array = np.copy(EMNumPy.em2numpy(self.reference_micrograph))
        return self.reference_micrograph_array

    def stack_reference_micrographs_to_scan(self):
        shifted_micrographs = {}
        horizontally_stacked_micrographs = {}
        for row_number in range(self.row_count):
            for column_number in range(self.column_count):
                shifted_micrograph_x = np.roll(self.reference_micrograph_array, 5*row_number)
                shifted_micrograph_xy = np.roll(shifted_micrograph_x, 5*column_number, axis=-2)
                shifted_micrographs['column{0}row{1}'.format(column_number, row_number)] = shifted_micrograph_xy 
                
            horizontally_stacked_micrographs['row{0}'.format(row_number)] = \
            np.hstack([shifted_micrographs['column{0}row{1}'.format(column_number, row_number)] for column_number in \
            range(self.column_count)])
    
        self.stacked_scan = np.vstack([horizontally_stacked_micrographs['row{0}'.format(row_number)] for row_number in \
        range(self.row_count)])
        
        self.scan = EMNumPy.numpy2em(np.copy(self.stacked_scan))

        return self.scan

    def prepare_noise_micrograph(self, scan_stepsize=1000.0, width=8.0, height=9, label_w=3.0, label_height=2.0):
#        self.micrograph_width_cm = 8
#        self.micrograph_height_cm = 9
#        self.label_width_cm = 3
#        self.label_height_cm = 2
#        self.scanner_stepsize_micrometer = 1000

#        self.reference_micrograph = self.make_refmic(self.micrograph_width_cm, self.micrograph_height_cm, self.label_width_cm, self.label_height_cm, self.scanner_stepsize_micrometer)
        self.reference_micrograph = ScanSplit().make_refmic(width, height, label_w, label_height, scan_stepsize)
        
        self.reference_micrograph = self.add_noise_to_reference_micrograph(self.reference_micrograph)
        
        return self.reference_micrograph
    

class TestScanSplit(ScanSplit, ScanPreparation):
    """
    Test class that tests work of scansplit
    """
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = ScanSplitPar()
        
        self.prepare_scan()
        
        self.feature_set.parameters['Micrographs']=self.scanned_micrograph
        self.feature_set.parameters['Output micrograph pattern']='test_split999.hdf'
        self.feature_set.parameters['Number of columns x rows']=(self.column_count, self.row_count)
        self.feature_set.parameters['Number of micrographs to be cropped']=self.number_of_micrographs
        
        self.feature_set.parameters['Micrograph width x height in cm']=(self.micrograph_width_cm,
        self.micrograph_height_cm)
        
        self.feature_set.parameters['Label width x height in cm']=(self.label_width_cm, self.label_height_cm)
        self.feature_set.parameters['Scanner step size in micrometer']=float(self.scanner_stepsize_micrometer)
        self.feature_set.parameters['Cross-correlation rejection criterion']=float(self.cross_correlation_criterion)
        self.feature_set.parameters['Normscan option']=True
        self.feature_set.parameters['Binning option']=True
        self.feature_set.parameters['Contact print option']=True
        self.feature_set.parameters['Final print option']=True
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)


        super(TestScanSplit, self).__init__(self.feature_set)

    
    def test_top_and_bottom_label_position(self):
        self.scan.write_image(self.scanned_micrograph)
        self.img = mirror(self.scan, axis='y')
        self.img = mirror(self.img, axis='x')

        self.perform_splitscan()

    def compare_shifts(self):
        micrograph_number = 0 
        for row_number in range(self.row_count):
            for column_number in range(self.column_count):
                sys.stderr.write('{0} = {1}\n'.format(self.xoffset[micrograph_number], 5*row_number))
                assert 5*row_number - 1 <= self.xoffset[micrograph_number] <= 5*row_number + 1
                sys.stderr.write('{0} = {1}\n'.format(self.yoffset[micrograph_number], 5*column_number))
                assert 5*column_number -1 <= self.yoffset[micrograph_number] <= 5*column_number + 1
                micrograph_number += 1

    def get_mean_and_sigma_of_reference_micrograph(self):
        self.reference_stat = Micrograph().get_statistics_from_image(self.reference_micrograph)

    def compare_mean_and_sigma_of_normalized_micrograph(self):
        """
        Test whether normalization worked
        """
        split_stat = Micrograph().get_statistics_from_image(self.micrograph)

        assert self.reference_stat.avg > split_stat.avg
        assert self.reference_stat.sigma < split_stat.sigma

    def prepare_split_micrographs_on_stack_for_gallery(self):
        files = glob('*test_split999_???.hdf')
        img = stackimg = EMData()
        for index, file in enumerate(files):
            img.read_image(file)
            stackimg.write_image('test_split999.hdf', index)
            
    def teardown(self):
        os.remove(self.scanned_micrograph)
        os.remove('test_scanmic_cprint.pdf')

        files = glob('*split999*')
        for each_file in files:
            os.remove(each_file)

        self.testingdir.remove()


class TestScanSplitMain(TestScanSplit):
    def do_test_case_ss1(self):
        self.get_mean_and_sigma_of_reference_micrograph()
        self.perform_splitscan()
        self.compare_mean_and_sigma_of_normalized_micrograph()
        self.prepare_split_micrographs_on_stack_for_gallery()


class TestScanSplitEndToEnd(TestScanSplit):
    def do_end_to_end_test_ss_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_ss_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)


class TestScanSplitMpi(TestScanSplit):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
        
def main():
    tss = TestScanSplitMain()
    tss.setup()
    tss.do_test_case_ss1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 18-Feb-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to plot refinement parameters from segmentrefine3d
"""
from PyQt4.QtGui import QApplication
from spring.csinfrastr.csdatabase import RefinementCycleTable, SpringDataBase, refine_base, SegmentTable, \
    RefinementCycleSegmentTable, RefinementCycleSegmentSubunitTable
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segmentplot import SegmentPlotPar, SegmentPlotGui, SegmentPlot
from spring.segment2d.segmentselect import SegmentSelect
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from sqlalchemy.sql.expression import desc
import numpy as np
import os
import shutil
import sys

class SegRefine3dPlotPar(SegmentPlotPar):
    """
    Class to initiate default dictionary with input parameters including help and range values and 
    status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'segrefine3dplot'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.segrefine3dplot_features = Features()
        self.feature_set = self.segrefine3dplot_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()
    
    def define_parameters_and_their_properties(self):
        self.feature_set = self.segrefine3dplot_features.set_inp_refinement_path(self.feature_set)
        self.feature_set = self.segrefine3dplot_features.set_interactive_vs_batch_mode(self.feature_set)
        self.feature_set = self.segrefine3dplot_features.set_diagnostic_prefix(self.feature_set, 'intermediate', 'Batch mode')
        self.feature_set = self.set_inp_spring_path(self.feature_set)
#        self.feature_set = self.set_refinement_cycle(self.feature_set)
        
        self.feature_set = self.set_ref_quantities_to_be_plotted(self.feature_set)
        self.feature_set = self.set_set_size(self.feature_set)
        
        self.feature_set = self.segrefine3dplot_features.set_micrograph_and_helix_selection_criteria(self.feature_set)
        

    def define_program_states(self):
        self.feature_set.program_states['get_ref_quantities_per_set']='Extract desired refinement quantities from spring database'
        self.feature_set.program_states['plot_data_on_figure']='Prepare figures with desired refined quantities'


    def set_refinement_cycle(self, feature_set):
        inp3 = 'Iteration cycle' 
        feature_set.parameters[inp3] = int(5)
        feature_set.hints[inp3] = 'Iteration cycle to be analyzed.'
        feature_set.properties[inp3] = feature_set.Range(1, 1000, 1)
        feature_set.level[inp3]='intermediate'
        
        return feature_set


    def set_ref_quantities_to_be_plotted(self, feature_set):
        inp3 = 'Refinement quantities'
        feature_set.parameters[inp3]=str('coordinates')
        feature_set.properties[inp3]=feature_set.choice_properties(2, ['coordinates', 'coordinates_subunit', 
        'in-plane_rotation', 'normalized_in-plane_rotation', 'out-of-plane_tilt', 'phi', 'theta', 'psi', 'x_shift', 
        'y_shift', 'shift_perpendicular_to_helix', 'shift_along_helix', 'ccc_peak'], 'QComboBox')
        
        feature_set.hints[inp3]=SpringDataBase().get_hints_from_ref_segment_table(feature_set.properties[inp3].choices)
        feature_set.level[inp3]='beginner'
        
        return feature_set
    
    
class SegRefine3dPlotPreparation(object):
    """
    * Class that holds functions for examining segments from micrographs

    * __init__ Function to interpret multi-input parameters

    """
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters

            self.infile = p['refinement.db file']
            self.outfile_prefix = p['Diagnostic plot prefix']
            self.spring_file = p['spring.db file']
            
            self.quantities = p['Refinement quantities']
            self.batch_mode = p['Batch mode']
            self.set_size = p['Set size']

            self = SegmentSelect().define_mics_and_helices_selection(self, p)


    def check_ref_values_and_raise_error(self, quantities, y_plot):
        if y_plot[0] is None:
            msg = '{0} entry does not exist in specified refinement.db. '.format(quantities) + \
            'Re-run segmentrefine3d to save relevant information.'
            raise ValueError(msg)


    def get_labels_according_to_ref_quantities(self, set_size, quantities):
        """
        >>> from spring.segment3d.segrefine3dplot import SegRefine3dPlot
        >>> SegRefine3dPlot().get_labels_according_to_ref_quantities('helix', 'coordinates') #doctest: +NORMALIZE_WHITESPACE
        ['Stack_id', 'Refined X-Coordinate (Angstrom)', 'Refined Y-Coordinate (Angstrom)', 
        'Picked X-Coordinate (Angstrom)', 'Picked Y-Coordinate (Angstrom)', 
        'Selected refined X-Coordinate (Angstrom)', 'Selected refined Y-Coordinate (Angstrom)']
        >>> SegRefine3dPlot().get_labels_according_to_ref_quantities('helix', 'in-plane_rotation') #doctest: +NORMALIZE_WHITESPACE
        ['Stack_id', 'Distance Along Helix (Angstrom)', 'In-Plane Rotation Angle (Degrees)', 
        'Selected In-Plane Rotation Angle (Degrees)', 'Local Average In-Plane Rotation Angle (Degrees)']
        >>> SegRefine3dPlot().get_labels_according_to_ref_quantities('helix', 'normalized_in-plane_rotation') #doctest: +NORMALIZE_WHITESPACE
        ['Stack_id', 'Distance Along Helix (Angstrom)', 'Normalized In-Plane Rotation Angle (Degrees)', 
        'Selected Normalized In-Plane Rotation Angle (Degrees)']
        >>> SegRefine3dPlot().get_labels_according_to_ref_quantities('helix', 'coordinates_subunit') #doctest: +NORMALIZE_WHITESPACE
        ['Stack_id', 'Refined subunit X-Coordinate (Angstrom)', 'Refined subunit Y-Coordinate (Angstrom)', 
        'Picked X-Coordinate (Angstrom)', 'Picked Y-Coordinate (Angstrom)', 'Selected refined X-Coordinate (Angstrom)', 
        'Selected refined Y-Coordinate (Angstrom)']
        """
        
        labels = ['Stack_id']
        if set_size == 'helix' and not quantities.startswith('coordinates'):
            labels += SpringDataBase().get_labels_from_table(SegmentTable, 'distance_from_start_A')
        elif quantities.startswith('coordinates'):
            coord_labels = SpringDataBase().get_labels_from_table(SegmentTable, 'x_coordinate_A',
            'y_coordinate_A')
            
            if quantities.endswith('coordinates'):
                ref_labels = ['Refined {0}'.format(each_coord_label) for each_coord_label in coord_labels]
            else:
                ref_labels = ['Refined subunit {0}'.format(each_coord_label) for each_coord_label in coord_labels]
            sel_labels = ['Selected refined {0}'.format(each_coord_label) for each_coord_label in coord_labels]
            coord_labels = ['Picked {0}'.format(each_coord_label) for each_coord_label in coord_labels]
            labels += ref_labels + coord_labels + sel_labels
            
        if quantities == 'phi':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'phi')
        elif quantities == 'theta':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'theta')
        elif quantities == 'psi':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'psi')
        elif quantities == 'in-plane_rotation':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'inplane_angle',
            'lavg_inplane_angle')
        elif quantities == 'normalized_in-plane_rotation':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'norm_inplane_angle')
        elif quantities == 'out-of-plane_tilt':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'out_of_plane_angle',
            'lavg_out_of_plane')
        elif quantities == 'x_shift':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'shift_x_A')
        elif quantities == 'y_shift':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'shift_y_A')
        elif quantities == 'shift_perpendicular_to_helix':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'helix_shift_x_A',
            'lavg_helix_shift_x_A')
        elif quantities == 'shift_along_helix':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'helix_shift_y_A')
        elif quantities == 'ccc_peak':
            labels += SpringDataBase().get_labels_from_table(RefinementCycleSegmentTable, 'peak')
            
        if set_size == 'helix' and not quantities.startswith('coordinates'):
            labels += ['Selected {0}'.format(labels[2])]
            
        if set_size == 'helix' and quantities in ['in-plane_rotation', 'out-of-plane_tilt',
        'shift_perpendicular_to_helix']:
            labels = labels[:-2] + [labels[-1]] + [labels[-2]]
            
        return labels
    
    
    def get_refinement_cycle_id(self, infile):
        shutil.copy(infile, os.path.basename(infile))
        
        ref_session = SpringDataBase().setup_sqlite_db(refine_base, os.path.basename(infile))

        last_cycle = ref_session.query(RefinementCycleTable).order_by(desc(RefinementCycleTable.id)).first()
            
        return ref_session, last_cycle
    

class SegRefine3dPlotDataRetrieval(SegRefine3dPlotPreparation):
    def get_x_and_y_coordinates_from_helix_segments(self, each_helix_segments, each_helix_ref_segments,
    excluded_segments):
        x_coord = np.array([each_helix_segment.picked_x_coordinate_A for each_helix_segment in each_helix_segments])
        y_coord = np.array([each_helix_segment.picked_y_coordinate_A for each_helix_segment in each_helix_segments])
        stack_ids = np.array([each_helix_segment.stack_id for each_helix_segment in each_helix_segments])
        
        ref_stack_ids = np.array([each_helix_ref_segment.stack_id for each_helix_ref_segment in each_helix_ref_segments])

        ref_all_x_shifts = np.zeros(len(stack_ids))
        ref_x_shifts = np.array([each_helix_ref_segment.shift_x_A for each_helix_ref_segment in each_helix_ref_segments])
        ref_all_x_shifts[stack_ids.searchsorted(ref_stack_ids)] = ref_x_shifts
        
        ref_x_coord = x_coord + ref_all_x_shifts

        ref_all_y_shifts = np.zeros(len(stack_ids))
        ref_y_shifts = np.array([each_helix_ref_segment.shift_y_A for each_helix_ref_segment in each_helix_ref_segments])
        ref_all_y_shifts[stack_ids.searchsorted(ref_stack_ids)] = ref_y_shifts
        
        ref_y_coord = y_coord + ref_all_y_shifts
#        ref_x_coord = x_coord + np.array([each_helix_ref_segment.shift_x_A for each_helix_ref_segment in
#        each_helix_ref_segments])
        
#        ref_y_coord = y_coord + np.array([each_helix_ref_segment.shift_y_A for each_helix_ref_segment in
#        each_helix_ref_segments])
        
        sel_x_coord = np.array(np.ma.masked_array(x_coord, mask=excluded_segments).tolist(), dtype=float)
        sel_y_coord = np.array(np.ma.masked_array(y_coord, mask=excluded_segments).tolist(), dtype=float)
        sel_ref_x_coord = np.array(np.ma.masked_array(ref_x_coord, mask=excluded_segments).tolist(), dtype=float)
        sel_ref_y_coord = np.array(np.ma.masked_array(ref_y_coord, mask=excluded_segments).tolist(), dtype=float)
        
        return x_coord, y_coord, ref_x_coord, ref_y_coord, sel_x_coord, sel_y_coord, sel_ref_x_coord, sel_ref_y_coord
    

    def get_refined_quantities_from_segments(self, quantities, each_helix_ref_segments):
        if quantities == 'normalized_in-plane_rotation':
            y_plot = [each_helix_ref_segment.norm_inplane_angle for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'phi':
            y_plot = [each_helix_ref_segment.phi for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'theta':
            y_plot = [each_helix_ref_segment.theta for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'psi':
            y_plot = [each_helix_ref_segment.psi for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'x_shift':
            y_plot = [each_helix_ref_segment.shift_x_A for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'y_shift':
            y_plot = [each_helix_ref_segment.shift_y_A for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'shift_along_helix':
            y_plot = [each_helix_ref_segment.helix_shift_y_A for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        elif quantities == 'ccc_peak':
            y_plot = [each_helix_ref_segment.peak for each_helix_ref_segment in each_helix_ref_segments]
            self.check_ref_values_and_raise_error(quantities, y_plot)
        else:
            y_plot = None
            
        return y_plot
    
    
    def get_refined_subunit_coordinates(self, session, ref_session, each_helix_ref_segments):
        sub_xcoord = np.array([])
        sub_ycoord = np.array([])
        subunit_stack_ids = np.array([])
        for each_ref_segment in each_helix_ref_segments:
            each_ref_subunits = ref_session.query(RefinementCycleSegmentSubunitTable).\
            filter(RefinementCycleSegmentSubunitTable.ref_seg_id == each_ref_segment.id).all()
        
            each_segment = session.query(SegmentTable).get(each_ref_segment.stack_id + 1)
        
            sub_xcoord = np.append(sub_xcoord, each_segment.picked_x_coordinate_A + \
            np.array([each_subunit.shift_x_A for each_subunit in each_ref_subunits]))
            
            sub_ycoord = np.append(sub_ycoord, each_segment.picked_y_coordinate_A + \
            np.array([each_subunit.shift_y_A for each_subunit in each_ref_subunits]))
        
            subunit_stack_ids = np.append(subunit_stack_ids, np.array([each_ref_segment.stack_id for each_subunit in
            each_ref_subunits]))
            
        return subunit_stack_ids, sub_xcoord, sub_ycoord
    

    def prepare_data_for_table_display(self, each_helix_stack_ids, x_coord, y_coord, sel_ref_x_coord, sel_ref_y_coord,
    each_helix_su_stack_ids, sub_xcoord, sub_ycoord):
    
        filled_x_coord = np.zeros(len(sub_xcoord))
        filled_y_coord = np.zeros(len(sub_ycoord))
        filled_ref_x_coord = np.zeros(len(sub_xcoord))
        filled_ref_y_coord = np.zeros(len(sub_ycoord))
        each_helix_stack_ids = np.array(each_helix_stack_ids)
        for each_unique_stack_id in each_helix_stack_ids:
            filled_x_coord[each_helix_su_stack_ids == each_unique_stack_id] = \
            x_coord[each_helix_stack_ids == each_unique_stack_id]
            
            filled_y_coord[each_helix_su_stack_ids == each_unique_stack_id] = \
            y_coord[each_helix_stack_ids == each_unique_stack_id]
            
            filled_ref_x_coord[each_helix_su_stack_ids == each_unique_stack_id] = \
            sel_ref_x_coord[each_helix_stack_ids == each_unique_stack_id]
            
            filled_ref_y_coord[each_helix_su_stack_ids == each_unique_stack_id] = \
            sel_ref_y_coord[each_helix_stack_ids == each_unique_stack_id]
        
        return each_helix_stack_ids, filled_x_coord, filled_y_coord, filled_ref_x_coord, filled_ref_y_coord


    def add_picked_coordinates_of_exluded_segments(self, each_helix_stack_ids, excluded_segments, x_coord, y_coord,
    each_helix_su_stack_ids, sub_xcoord, sub_ycoord, filled_x_coord, filled_y_coord, filled_ref_x_coord,
    filled_ref_y_coord):
        
        excl_x_coord = np.ma.masked_array(x_coord, mask=np.invert(excluded_segments)).compressed()
        excl_y_coord = np.ma.masked_array(y_coord, mask=np.invert(excluded_segments)).compressed()
        excl_stack_ids = np.ma.masked_array(each_helix_stack_ids, mask=np.invert(excluded_segments)).compressed()
        for each_ex_id, each_ex_refx, each_ex_refy in zip(excl_stack_ids, excl_x_coord, excl_y_coord):
            each_helix_su_stack_ids = np.append(each_helix_su_stack_ids, each_ex_id)
            sub_xcoord = np.append(sub_xcoord, np.nan)
            sub_ycoord = np.append(sub_ycoord, np.nan)
            filled_x_coord = np.append(filled_x_coord, each_ex_refx)
            filled_y_coord = np.append(filled_y_coord, each_ex_refy)
            filled_ref_x_coord = np.append(filled_ref_x_coord, np.nan)
            filled_ref_y_coord = np.append(filled_ref_y_coord, np.nan)
        
        return each_helix_su_stack_ids, sub_xcoord, sub_ycoord, filled_x_coord, filled_y_coord, filled_ref_x_coord, filled_ref_y_coord


    def get_coordinates_from_refinement(self, session, ref_session, each_plot_set_string, quantities, xy_quantities,
    each_helix_stack_ids, each_helix_segments, each_helix_ref_segments, excluded_segments):
    
        x_coord, y_coord, ref_x_coord, ref_y_coord, sel_x_coord, sel_y_coord, sel_ref_x_coord, sel_ref_y_coord = \
        self.get_x_and_y_coordinates_from_helix_segments(each_helix_segments, each_helix_ref_segments,
        excluded_segments)
            
        if quantities == 'coordinates':
            self.check_ref_values_and_raise_error(quantities, y_coord)
            
            xy_quantities.append((each_plot_set_string, np.vstack([each_helix_stack_ids, ref_x_coord, ref_y_coord,
            x_coord, y_coord, sel_ref_x_coord, sel_ref_y_coord])))
            
        elif quantities == 'coordinates_subunit':
            each_helix_su_stack_ids, sub_xcoord, sub_ycoord = self.get_refined_subunit_coordinates(session, ref_session,
            each_helix_ref_segments)
            
            each_helix_stack_ids, filled_x_coord, filled_y_coord, filled_ref_x_coord, filled_ref_y_coord = \
            self.prepare_data_for_table_display(each_helix_stack_ids, x_coord, y_coord, sel_ref_x_coord,
            sel_ref_y_coord, each_helix_su_stack_ids, sub_xcoord, sub_ycoord)
                
            each_helix_su_stack_ids, sub_xcoord, sub_ycoord, filled_x_coord, filled_y_coord, filled_ref_x_coord, \
            filled_ref_y_coord = self.add_picked_coordinates_of_exluded_segments(each_helix_stack_ids,
            excluded_segments, x_coord, y_coord, each_helix_su_stack_ids, sub_xcoord, sub_ycoord, filled_x_coord,
            filled_y_coord, filled_ref_x_coord, filled_ref_y_coord)
                
            xy_quantities.append((each_plot_set_string, np.vstack([each_helix_su_stack_ids, sub_xcoord, sub_ycoord,
            filled_x_coord, filled_y_coord, filled_ref_x_coord, filled_ref_y_coord])))
        return xy_quantities


    def get_refinement_parameters_no_local_averaging(self, each_plot_set_string, set_size, quantities, xy_quantities,
    each_helix_stack_ids, each_helix_ref_segments, excluded_segments, distances):
        
        if quantities in ['phi', 'theta', 'psi', 'x_shift', 'y_shift', 'normalized_in-plane_rotation',
        'shift_along_helix', 'ccc_peak']:
            y_plot = self.get_refined_quantities_from_segments(quantities, each_helix_ref_segments)
            sel_y_plot = np.array(np.ma.masked_array(y_plot, mask=excluded_segments).tolist(), dtype=float)
            if set_size == 'helix':
                xy_quantities.append((each_plot_set_string, np.vstack([each_helix_stack_ids, distances, y_plot,
                sel_y_plot])))
            else:
                xy_quantities.append((each_plot_set_string, np.vstack([each_helix_stack_ids, y_plot])))
                
        return xy_quantities
    

    def get_refinement_parameters_including_local_averaging(self, each_plot_set_string, set_size, quantities,
    xy_quantities, each_helix_stack_ids, each_helix_ref_segments, excluded_segments, each_helix_ref_segment, distances):
        
        if quantities in ['in-plane_rotation', 'out-of-plane_tilt', 'shift_perpendicular_to_helix']:
            if quantities == 'in-plane_rotation':
                y_plot = [each_helix_ref_segment.inplane_angle for each_helix_ref_segment in each_helix_ref_segments]
                
                avg_y_plot = [each_helix_ref_segment.lavg_inplane_angle for each_helix_ref_segment in \
                each_helix_ref_segments]
                
                self.check_ref_values_and_raise_error(quantities, y_plot)
            elif quantities == 'out-of-plane_tilt':
                y_plot = [each_helix_ref_segment.out_of_plane_angle for each_helix_ref_segment in each_helix_ref_segments]
                
                avg_y_plot = [each_helix_ref_segment.lavg_out_of_plane for each_helix_ref_segment in \
                each_helix_ref_segments]
                
                self.check_ref_values_and_raise_error(quantities, y_plot)
            elif quantities == 'shift_perpendicular_to_helix':
                y_plot = [each_helix_ref_segment.helix_shift_x_A for each_helix_ref_segment in each_helix_ref_segments]
                
                avg_y_plot = [each_helix_ref_segment.lavg_helix_shift_x_A for each_helix_ref_segment in \
                each_helix_ref_segments]
                
                self.check_ref_values_and_raise_error(quantities, y_plot)
            sel_y_plot = np.array(np.ma.masked_array(y_plot, mask=excluded_segments).tolist(), dtype=float)
            if set_size == 'helix':
                xy_quantities.append((each_plot_set_string, np.vstack([each_helix_stack_ids, distances, y_plot,
                sel_y_plot, avg_y_plot])))
            else:
                xy_quantities.append((each_plot_set_string, np.vstack([each_helix_stack_ids, y_plot])))
            
        return xy_quantities
    

    def get_refinement_parameters(self, each_plot_set_string, set_size, quantities, xy_quantities, each_helix_stack_ids,
    each_helix_segments, each_helix_ref_segments, excluded_segments, each_helix_ref_segment, distances):
        
        if quantities in ['phi', 'theta', 'psi', 'x_shift', 'y_shift', 'normalized_in-plane_rotation',
        'shift_along_helix', 'ccc_peak', 'in-plane_rotation', 'out-of-plane_tilt', 'shift_perpendicular_to_helix']:
            
            xy_quantities = self.get_refinement_parameters_no_local_averaging(each_plot_set_string, set_size,
            quantities, xy_quantities, each_helix_stack_ids, each_helix_ref_segments, excluded_segments, distances)
            
            xy_quantities = self.get_refinement_parameters_including_local_averaging(each_plot_set_string, set_size,
            quantities, xy_quantities, each_helix_stack_ids, each_helix_ref_segments, excluded_segments,
            each_helix_ref_segment, distances)
        
        return xy_quantities
    

    def get_specified_ref_quantities(self, session, ref_session, each_plot_set_string, set_size, quantities, xy_quantities,
    each_helix_stack_ids, each_helix_segments, each_helix_ref_segments):
    
        ref_stack_ids = np.array([each_helix_ref_segment.stack_id for each_helix_ref_segment in each_helix_ref_segments])
        
        excluded_ref_segments = np.invert([bool(each_helix_ref_segment.selected) for each_helix_ref_segment in \
        each_helix_ref_segments])
        
        helix_stack_ids = np.array(each_helix_stack_ids)
        excluded_segments = np.array(len(each_helix_stack_ids) * [False])
        excluded_segments[helix_stack_ids.searchsorted(ref_stack_ids)] = excluded_ref_segments
        
        xy_quantities = self.get_coordinates_from_refinement(session, ref_session, each_plot_set_string, quantities,
        xy_quantities, each_helix_stack_ids, each_helix_segments, each_helix_ref_segments, excluded_segments)
            
        distances = np.array([each_helix_segment.distance_from_start_A for each_helix_segment in each_helix_segments])
        ref_distances = distances[helix_stack_ids.searchsorted(ref_stack_ids)] 
        
        xy_quantities = self.get_refinement_parameters(each_plot_set_string, set_size, quantities, xy_quantities,
        ref_stack_ids, each_helix_segments, each_helix_ref_segments, excluded_ref_segments,
        each_helix_ref_segment, ref_distances)
                
        return xy_quantities
    

    def get_ref_quantities_per_set(self, spring_path, quantities):
        self.log.fcttolog()
        shutil.copy(spring_path, 'spring.db')
        
        session, plot_sets = SegmentPlot().get_plot_set(self.set_size, self)
        combined_included_segments = SegmentPlot().filter_segments_by_helix_and_micrograph_criteria(session, self)
        
        labels = self.get_labels_according_to_ref_quantities(self.set_size, quantities)
        ref_session, ref_cycle = self.get_refinement_cycle_id(self.infile)
        xy_quantities = []
        for each_plot_set in plot_sets:
            each_helix_seg_ids, each_helix_segments = SegmentPlot().get_segment_ids_according_to_set_size(session,
            self.set_size, each_plot_set)
            
            if len(set(each_helix_seg_ids).intersection(combined_included_segments)) != 0:
                each_set_id_label = SegmentPlot().generate_set_id_label_for_plot_and_filename(self.set_size,
                each_plot_set)
                
                each_helix_ref_segments = SegmentRefine3d().get_all_segments_from_refinement_cycle(ref_session,
                ref_cycle, each_helix_seg_ids)
                
                xy_quantities = self.get_specified_ref_quantities(session, ref_session, each_set_id_label, self.set_size, quantities,
                xy_quantities, each_helix_seg_ids, each_helix_segments, each_helix_ref_segments)
                        
        log_info = SegmentPlot().prepare_log_string(labels, xy_quantities)
                
        self.log.ilog(log_info)
                    
        return labels, xy_quantities
    
    
class SegRefine3dPlotVisualize(SegRefine3dPlotDataRetrieval):
    def plot_ref_data_on_figure(self, xy_quantities, quantities, labels):
        segplot = SegmentPlot()
        segplot.feature_set = self.feature_set
        segplot.batch_mode = self.batch_mode
        segplot.infile = self.infile
        segplot.outfile_prefix = self.outfile_prefix
        segplot.set_size = self.set_size
        if hasattr(self, 'pixelsize'):
            segplot.pixelsize = self.pixelsize
        
        figures = segplot.plot_data_on_figure(xy_quantities, labels)

        return figures, segplot
            
            
class SegRefine3dPlot(SegRefine3dPlotVisualize):
        
    def launch_interactive_plot_gui(self, figures, tables, labels):
        labels = [labels for each_table in tables]
        app = QApplication(sys.argv)
        symexplor = SegmentPlotGui(self.feature_set, figures, tables, labels)
        symexplor.show()
        app.exec_()

        
    def plot_desired_ref_quantities(self):
        labels, xy_quantities = self.get_ref_quantities_per_set(self.spring_file, self.quantities)
        self.log.plog(10)
        figures, segplot = self.plot_ref_data_on_figure(xy_quantities, self.quantities, labels)
        if not self.batch_mode:
            all_labels = [labels for each_table in xy_quantities]
            segplot.launch_interactive_plot_gui(self.feature_set, figures, xy_quantities, all_labels)
        
        self.log.endlog(self.feature_set)
        
        
def main():
    # Option handling
    parset = SegRefine3dPlotPar()
    mergeparset = OptHandler(parset)
    ######## Program
    stack = SegRefine3dPlot(mergeparset)
    stack.plot_desired_ref_quantities()

if __name__ == '__main__':
    main()
